# Copyright 2013-2020 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

import itertools

from portage.tests import TestCase
from portage.tests.resolver.ResolverPlayground import (ResolverPlayground,
	ResolverPlaygroundTestCase)

class OrChoicesTestCase(TestCase):

	def testOrChoices(self):
		ebuilds = {
			"dev-lang/vala-0.20.0" : {
				"EAPI": "5",
				"SLOT": "0.20"
			},
			"dev-lang/vala-0.18.0" : {
				"EAPI": "5",
				"SLOT": "0.18"
			},
			#"dev-libs/gobject-introspection-1.36.0" : {
			#	"EAPI": "5",
			#	"RDEPEND" : "!<dev-lang/vala-0.20.0",
			#},
			"dev-libs/gobject-introspection-1.34.0" : {
				"EAPI": "5"
			},
			"sys-apps/systemd-ui-2" : {
				"EAPI": "5",
				"RDEPEND" : "|| ( dev-lang/vala:0.20 dev-lang/vala:0.18 )"
			},
		}

		installed = {
			"dev-lang/vala-0.18.0" : {
				"EAPI": "5",
				"SLOT": "0.18"
			},
			"dev-libs/gobject-introspection-1.34.0" : {
				"EAPI": "5"
			},
			"sys-apps/systemd-ui-2" : {
				"EAPI": "5",
				"RDEPEND" : "|| ( dev-lang/vala:0.20 dev-lang/vala:0.18 )"
			},
		}

		world = ["dev-libs/gobject-introspection", "sys-apps/systemd-ui"]

		test_cases = (
			# Demonstrate that vala:0.20 update is pulled in, for bug #478188
			ResolverPlaygroundTestCase(
				["@world"],
				options = {"--update": True, "--deep": True},
				success=True,
				all_permutations = True,
				mergelist = ['dev-lang/vala-0.20.0']),
			# Verify that vala:0.20 is not pulled in without --deep
			ResolverPlaygroundTestCase(
				["@world"],
				options = {"--update": True},
				success=True,
				all_permutations = True,
				mergelist = []),
			# Verify that vala:0.20 is not pulled in without --update
			ResolverPlaygroundTestCase(
				["@world"],
				options = {"--selective": True, "--deep": True},
				success=True,
				all_permutations = True,
				mergelist = []),
		)

		playground = ResolverPlayground(ebuilds=ebuilds, installed=installed, world=world)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True, test_case.fail_msg)
		finally:
			playground.cleanup()

	def testInitiallyUnsatisfied(self):

		ebuilds = {

			"app-misc/A-1" : {
				"EAPI": "5",
				"SLOT": "0/1"
			},

			"app-misc/A-2" : {
				"EAPI": "5",
				"SLOT": "0/2"
			},

			"app-misc/B-0" : {
				"EAPI": "5",
				"RDEPEND": "app-misc/A:="
			},

			"app-misc/C-0" : {
				"EAPI": "5",
				"RDEPEND": "|| ( app-misc/X <app-misc/A-2 )"
			},

		}

		installed = {

			"app-misc/A-1" : {
				"EAPI": "5",
				"SLOT": "0/1"
			},

			"app-misc/B-0" : {
				"EAPI": "5",
				"RDEPEND": "app-misc/A:0/1="
			},

			"app-misc/C-0" : {
				"EAPI": "5",
				"RDEPEND": "|| ( app-misc/X <app-misc/A-2 )"
			},

		}

		world = ["app-misc/B", "app-misc/C"]

		test_cases = (

			# Test bug #522652, where the unsatisfiable app-misc/X
			# atom is selected, and the dependency is placed into
			# _initially_unsatisfied_deps where it is ignored, causing
			# upgrade to app-misc/A-2 (breaking a dependency of
			# app-misc/C-0).
			ResolverPlaygroundTestCase(
				["app-misc/A"],
				options = {},
				success = True,
				mergelist = ['app-misc/A-1']
			),

		)

		playground = ResolverPlayground(ebuilds=ebuilds,
			installed=installed, world=world, debug=False)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True, test_case.fail_msg)
		finally:
			playground.cleanup()


	def testUseMask(self):

		profile = {
			"use.mask":
			(
				"abi_ppc_32",
			),
		}

		ebuilds = {

			"sys-libs/A-1" : {
				"EAPI": "5",
				"RDEPEND": "|| ( sys-libs/zlib[abi_ppc_32(-)] " + \
					"sys-libs/zlib[abi_x86_32(-)] )"
			},

			"sys-libs/zlib-1.2.8-r1" : {
				"EAPI": "5",
				"IUSE": "abi_ppc_32 abi_x86_32"
			},

			"sys-libs/zlib-1.2.8" : {
				"EAPI": "5",
				"IUSE": ""
			},
		}

		test_cases = (

			# bug #515584: We want to prefer choices that do
			# not require changes to use.mask or use.force.
			# In this case, abi_ppc_32 is use.masked in the
			# profile, so we want to avoid that choice.
			ResolverPlaygroundTestCase(
				["sys-libs/A"],
				options = {},
				success = False,
				use_changes = {
					'sys-libs/zlib-1.2.8-r1': {'abi_x86_32': True}
				},
				mergelist = ["sys-libs/zlib-1.2.8-r1", "sys-libs/A-1"]
			),

		)

		playground = ResolverPlayground(ebuilds=ebuilds,
			profile=profile, debug=False)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True,
					test_case.fail_msg)
		finally:
			playground.cleanup()

	def testConflictMissedUpdate(self):

		ebuilds = {
			"dev-lang/ocaml-4.02.1" : {
				"EAPI": "5",
				"SLOT": "0/4.02.1",
			},

			"dev-lang/ocaml-4.01.0" : {
				"EAPI": "5",
				"SLOT": "0/4.01.0",
			},

			"dev-ml/lablgl-1.05" : {
				"EAPI": "5",
				"DEPEND": (">=dev-lang/ocaml-3.10.2:= "
					"|| ( dev-ml/labltk:= <dev-lang/ocaml-4.02 )"),
				"RDEPEND": (">=dev-lang/ocaml-3.10.2:= "
					"|| ( dev-ml/labltk:= <dev-lang/ocaml-4.02 )"),
			},

			"dev-ml/labltk-8.06.0" : {
				"EAPI": "5",
				"SLOT": "0/8.06.0",
				"DEPEND": ">=dev-lang/ocaml-4.02:=",
				"RDEPEND": ">=dev-lang/ocaml-4.02:=",
			},
		}

		installed = {
			"dev-lang/ocaml-4.01.0" : {
				"EAPI": "5",
				"SLOT": "0/4.01.0",
			},

			"dev-ml/lablgl-1.05" : {
				"EAPI": "5",
				"DEPEND": (">=dev-lang/ocaml-3.10.2:0/4.01.0= "
					"|| ( dev-ml/labltk:= <dev-lang/ocaml-4.02 )"),
				"RDEPEND": (">=dev-lang/ocaml-3.10.2:0/4.01.0= "
					"|| ( dev-ml/labltk:= <dev-lang/ocaml-4.02 )"),
			},
		}

		world = (
			"dev-lang/ocaml",
			"dev-ml/lablgl",
		)

		test_cases = (

			# bug #531656: If an ocaml update is desirable,
			# then we need to pull in dev-ml/labltk.
			ResolverPlaygroundTestCase(
				["@world"],
				options = {"--update": True, "--deep": True},
				success = True,
				mergelist = [
					"dev-lang/ocaml-4.02.1",
					"dev-ml/labltk-8.06.0",
					"dev-ml/lablgl-1.05",
				]
			),

		)

		playground = ResolverPlayground(debug=False,
			ebuilds=ebuilds, installed=installed, world=world)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True,
					test_case.fail_msg)
		finally:
			# Disable debug so that cleanup works.
			playground.debug = False
			playground.cleanup()

	def test_python_slot(self):
		ebuilds = {

			"dev-lang/python-3.8" : {
				"EAPI": "7",
				"SLOT": "3.8"
			},

			"dev-lang/python-3.7" : {
				"EAPI": "7",
				"SLOT": "3.7"
			},

			"dev-lang/python-3.6" : {
				"EAPI": "7",
				"SLOT": "3.6"
			},

			"app-misc/bar-1" : {
				"EAPI": "7",
				"IUSE": "python_targets_python3_6 +python_targets_python3_7",
				"RDEPEND": "python_targets_python3_7? ( dev-lang/python:3.7 ) python_targets_python3_6? ( dev-lang/python:3.6 )"
			},

			"app-misc/foo-1" : {
				"EAPI": "7",
				"RDEPEND": "|| ( dev-lang/python:3.8 dev-lang/python:3.7 dev-lang/python:3.6 )"
			},

		}

		installed = {

			"dev-lang/python-3.7" : {
				"EAPI": "7",
				"SLOT": "3.7"
			},

			"app-misc/bar-1" : {
				"EAPI": "7",
				"IUSE": "python_targets_python3_6 +python_targets_python3_7",
				"USE": "python_targets_python3_7",
				"RDEPEND": "dev-lang/python:3.7"
			},

			"app-misc/foo-1" : {
				"EAPI": "7",
				"RDEPEND": "|| ( dev-lang/python:3.8 dev-lang/python:3.7 dev-lang/python:3.6 )"
			},

		}

		world = ["app-misc/foo", "app-misc/bar"]

		test_cases = (

			ResolverPlaygroundTestCase(
				["@world"],
				options = {"--update": True, "--deep": True},
				success = True,
				mergelist = ['dev-lang/python-3.8']
			),

		)

		playground = ResolverPlayground(ebuilds=ebuilds,
			installed=installed, world=world, debug=False)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True, test_case.fail_msg)
		finally:
			playground.debug = False
			playground.cleanup()

		installed = {

			"dev-lang/python-3.8" : {
				"EAPI": "7",
				"SLOT": "3.8"
			},

			"dev-lang/python-3.7" : {
				"EAPI": "7",
				"SLOT": "3.7"
			},

			"app-misc/bar-1" : {
				"EAPI": "7",
				"IUSE": "python_targets_python3_6 +python_targets_python3_7",
				"USE": "python_targets_python3_7",
				"RDEPEND": "dev-lang/python:3.7"
			},

			"app-misc/foo-1" : {
				"EAPI": "7",
				"RDEPEND": "|| ( dev-lang/python:3.8 dev-lang/python:3.7 dev-lang/python:3.6 )"
			},

		}

		test_cases = (
			# Test for bug 707108, where a new python slot was erroneously
			# removed by emerge --depclean.
			ResolverPlaygroundTestCase(
				[],
				options={"--depclean": True},
				success=True,
				cleanlist=[],
			),
		)

		playground = ResolverPlayground(ebuilds=ebuilds,
			installed=installed, world=world, debug=False)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True, test_case.fail_msg)
		finally:
			playground.debug = False
			playground.cleanup()

	def test_virtual_w3m(self):
		ebuilds = {

			'app-text/xmlto-0.0.28-r1' : {
				'EAPI': '7',
				'RDEPEND': '|| ( virtual/w3m www-client/lynx www-client/elinks )'
			},

			'www-client/elinks-0.13_pre_pre20180225' : {
				'EAPI': '7',
			},

			'www-client/lynx-2.9.0_pre4' : {
				'EAPI': '7',
			},

			'virtual/w3m-0' : {
				'EAPI': '7',
				'RDEPEND': '|| ( www-client/w3m www-client/w3mmee )'
			},

			'www-client/w3m-0.5.3_p20190105' : {
				'EAPI': '7',
			},

			'www-client/w3mmee-0.3.2_p24-r10' : {
				'EAPI': '7',
			},

		}

		installed = {

			'app-text/xmlto-0.0.28-r1' : {
				'EAPI': '7',
				'RDEPEND': '|| ( virtual/w3m www-client/lynx www-client/elinks )'
			},

			'www-client/elinks-0.13_pre_pre20180225' : {
				'EAPI': '7',
			},

			'www-client/lynx-2.9.0_pre4' : {
				'EAPI': '7',
			},

		}

		world = ['app-text/xmlto', 'www-client/elinks', 'www-client/lynx']

		test_cases = (

			# Test for bug 649622 (without www-client/w3m installed),
			# where virtual/w3m was pulled in only to be removed by the
			# next emerge --depclean.
			ResolverPlaygroundTestCase(
				['@world'],
				options = {'--update': True, '--deep': True},
				success = True,
				mergelist = []
			),

		)

		playground = ResolverPlayground(ebuilds=ebuilds,
			installed=installed, world=world, debug=False)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True, test_case.fail_msg)
		finally:
			playground.debug = False
			playground.cleanup()

		installed = dict(itertools.chain(installed.items(), {

			'www-client/w3m-0.5.3_p20190105' : {
				'EAPI': '7',
			},

		}.items()))

		test_cases = (

			# Test for bug 649622 (with www-client/w3m installed),
			# where virtual/w3m was pulled in only to be removed by the
			# next emerge --depclean.
			ResolverPlaygroundTestCase(
				['@world'],
				options = {'--update': True, '--deep': True},
				success = True,
				mergelist = []
			),

		)

		playground = ResolverPlayground(ebuilds=ebuilds,
			installed=installed, world=world, debug=False)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True, test_case.fail_msg)
		finally:
			playground.debug = False
			playground.cleanup()

		installed = dict(itertools.chain(installed.items(), {

			'virtual/w3m-0' : {
				'EAPI': '7',
				'RDEPEND': '|| ( www-client/w3m www-client/w3mmee )'
			},

		}.items()))

		test_cases = (

			# Test for bug 649622, where virtual/w3m is removed by
			# emerge --depclean immediately after it's installed
			# by a world update. Note that removal of virtual/w3m here
			# is essentially indistinguishable from removal of
			# dev-util/cmake-bootstrap in the depclean test case for
			# bug 703440.
			ResolverPlaygroundTestCase(
				[],
				options={'--depclean': True},
				success=True,
				cleanlist=['virtual/w3m-0', 'www-client/w3m-0.5.3_p20190105'],
			),

		)

		playground = ResolverPlayground(ebuilds=ebuilds,
			installed=installed, world=world, debug=False)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True, test_case.fail_msg)
		finally:
			playground.debug = False
			playground.cleanup()


		test_cases = (

			# Test for behavior reported in bug 649622 comment #10, where
			# depclean removed virtual/w3m-0 even though www-client/w3m
			# was in the world file. Since nothing is removed here, it
			# means that we have not reproduced the behavior reported in
			# this comment.
			ResolverPlaygroundTestCase(
				[],
				options={'--depclean': True},
				success=True,
				cleanlist=[],
			),

		)

		world += ['www-client/w3m']

		playground = ResolverPlayground(ebuilds=ebuilds,
			installed=installed, world=world, debug=False)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True, test_case.fail_msg)
		finally:
			playground.debug = False
			playground.cleanup()


	def test_virtual_w3m_realistic(self):
		"""
		Test for bug 649622 with realistic www-client/w3m dependencies copied
		from real ebuilds.
		"""
		ebuilds = {

			'app-misc/neofetch-6.1.0': {
				'EAPI': '7',
				'RDEPEND': 'www-client/w3m'
			},

			'app-text/xmlto-0.0.28-r1' : {
				'EAPI': '7',
				'RDEPEND': '|| ( virtual/w3m www-client/lynx www-client/elinks )'
			},

			'mail-client/neomutt-20191207': {
				'EAPI': '7',
				'RDEPEND': '|| ( www-client/lynx www-client/w3m www-client/elinks )'
			},

			'www-client/elinks-0.13_pre_pre20180225' : {
				'EAPI': '7',
			},

			'www-client/lynx-2.9.0_pre4' : {
				'EAPI': '7',
			},

			'virtual/w3m-0' : {
				'EAPI': '7',
				'RDEPEND': '|| ( www-client/w3m www-client/w3mmee )'
			},

			'www-client/w3m-0.5.3_p20190105' : {
				'EAPI': '7',
			},

			'www-client/w3mmee-0.3.2_p24-r10' : {
				'EAPI': '7',
			},

			'x11-base/xorg-server-1.20.7' : {
				'EAPI': '7',
				'RDEPEND': '|| ( www-client/links www-client/lynx www-client/w3m ) app-text/xmlto',
			}
		}

		installed = {

			'app-misc/neofetch-6.1.0': {
				'EAPI': '7',
				'RDEPEND': 'www-client/w3m'
			},

			'app-text/xmlto-0.0.28-r1' : {
				'EAPI': '7',
				'RDEPEND': '|| ( virtual/w3m www-client/lynx www-client/elinks )'
			},

			'mail-client/neomutt-20191207': {
				'EAPI': '7',
				'RDEPEND': '|| ( www-client/lynx www-client/w3m www-client/elinks )'
			},

			'www-client/lynx-2.9.0_pre4' : {
				'EAPI': '7',
			},

			'www-client/w3m-0.5.3_p20190105' : {
				'EAPI': '7',
			},

			'x11-base/xorg-server-1.20.7' : {
				'EAPI': '7',
				'RDEPEND': '|| ( www-client/links www-client/lynx www-client/w3m ) app-text/xmlto',
			}
		}

		world = ['app-misc/neofetch', 'mail-client/neomutt', 'www-client/lynx', 'x11-base/xorg-server']

		test_cases = (

			# Test for bug 649622 (with www-client/w3m installed via
			# xorg-server dependency), where virtual/w3m was pulled in
			# only to be removed by the next emerge --depclean. Note
			# that graph_order must be deterministic in order to achieve
			# deterministic results which are consistent between both
			# update and removal (depclean) actions.
			ResolverPlaygroundTestCase(
				['@world'],
				options = {'--update': True, '--deep': True},
				success = True,
				mergelist=['virtual/w3m-0'],
				graph_order=['@world', '@profile', '@selected', '@system', '[nomerge]app-misc/neofetch-6.1.0', '[nomerge]mail-client/neomutt-20191207', '[nomerge]www-client/lynx-2.9.0_pre4', '[nomerge]x11-base/xorg-server-1.20.7', '[nomerge]app-text/xmlto-0.0.28-r1', '[nomerge]www-client/w3m-0.5.3_p20190105', 'virtual/w3m-0'],
			),

		)

		playground = ResolverPlayground(ebuilds=ebuilds,
			installed=installed, world=world, debug=False)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True, test_case.fail_msg)
		finally:
			playground.debug = False
			playground.cleanup()


		installed = dict(itertools.chain(installed.items(), {

			'virtual/w3m-0' : {
				'EAPI': '7',
				'RDEPEND': '|| ( www-client/w3m www-client/w3mmee )'
			},

		}.items()))

		test_cases = (

			# Test for bug 649622, where virtual/w3m is removed by
			# emerge --depclean immediately after it's installed
			# by a world update. Since virtual/w3m-0 is not removed
			# here, this case fails to reproduce bug 649622. Note
			# that graph_order must be deterministic in order to achieve
			# deterministic results which are consistent between both
			# update and removal (depclean) actions.
			ResolverPlaygroundTestCase(
				[],
				options={'--depclean': True},
				success=True,
				cleanlist=[],
				graph_order=['@world', '@____depclean_protected_set____', '@profile', '@selected', '@system', '[nomerge]app-misc/neofetch-6.1.0', '[nomerge]mail-client/neomutt-20191207', '[nomerge]www-client/lynx-2.9.0_pre4', '[nomerge]x11-base/xorg-server-1.20.7', '[nomerge]app-text/xmlto-0.0.28-r1', '[nomerge]www-client/w3m-0.5.3_p20190105', '[nomerge]virtual/w3m-0'],
			),

		)

		playground = ResolverPlayground(ebuilds=ebuilds,
			installed=installed, world=world, debug=False)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True, test_case.fail_msg)
		finally:
			playground.debug = False
			playground.cleanup()


class OrChoicesLibpostprocTestCase(TestCase):

	def testOrChoicesLibpostproc(self):
		# This test case is expected to fail after the fix for bug 706278,
		# since the "undesirable" slot upgrade which triggers a blocker conflict
		# in this test case is practically indistinguishable from a desirable
		# slot upgrade. This particular blocker conflict is no longer relevant,
		# since current versions of media-libs/libpostproc are no longer
		# compatible with any available media-video/ffmpeg slot. In order to
		# solve this test case, some fancy backtracking (like for bug 382421)
		# will be required.
		self.todo = True

		ebuilds = {
			"media-video/ffmpeg-0.10" : {
				"EAPI": "5",
				"SLOT": "0.10"
			},
			"media-video/ffmpeg-1.2.2" : {
				"EAPI": "5",
				"SLOT": "0"
			},
			"media-libs/libpostproc-0.8.0.20121125" : {
				"EAPI": "5"
			},
			"media-plugins/gst-plugins-ffmpeg-0.10.13_p201211-r1" : {
				"EAPI": "5",
				"RDEPEND" : "|| ( media-video/ffmpeg:0 media-libs/libpostproc )"
			},
		}

		installed = {
			"media-video/ffmpeg-0.10" : {
				"EAPI": "5",
				"SLOT": "0.10"
			},
			"media-libs/libpostproc-0.8.0.20121125" : {
				"EAPI": "5"
			},
			"media-plugins/gst-plugins-ffmpeg-0.10.13_p201211-r1" : {
				"EAPI": "5",
				"RDEPEND" : "|| ( media-video/ffmpeg:0 media-libs/libpostproc )"
			},
		}

		world = ["media-plugins/gst-plugins-ffmpeg"]

		test_cases = (
			# Demonstrate that libpostproc is preferred
			# over ffmpeg:0 for bug #480736.
			ResolverPlaygroundTestCase(
				["@world"],
				options = {"--update": True, "--deep": True},
				success=True,
				all_permutations = True,
				mergelist = []),
		)

		playground = ResolverPlayground(ebuilds=ebuilds, installed=installed,
			world=world, debug=False)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True, test_case.fail_msg)
		finally:
			playground.debug = False
			playground.cleanup()



# Copyright 2013 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

import stat

from portage import os
from portage.exception import PermissionDenied

def exists_raise_eaccess(path):
	try:
		os.stat(path)
	except OSError as e:
		if e.errno == PermissionDenied.errno:
			raise PermissionDenied("stat('%s')" % path)
		return False
	else:
		return True

def isdir_raise_eaccess(path):
	try:
		st = os.stat(path)
	except OSError as e:
		if e.errno == PermissionDenied.errno:
			raise PermissionDenied("stat('%s')" % path)
		return False
	else:
		return stat.S_ISDIR(st.st_mode)



# test_dep_getusedeps.py -- Portage Unit Testing Functionality
# Copyright 2007-2010 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

from portage.tests import TestCase
from portage.dep import dep_getusedeps

from portage.tests import test_cps, test_slots, test_versions, test_usedeps

class DepGetUseDeps(TestCase):
	""" A simple testcase for dep_getusedeps
	"""

	def testDepGetUseDeps(self):

		for mycpv in test_cps:
			for version in test_versions:
				for slot in test_slots:
					for use in test_usedeps:
						cpv = mycpv[:]
						if version:
							cpv += version
						if slot:
							cpv += ":" + slot
						if isinstance(use, tuple):
							cpv += "[%s]" % (",".join(use),)
							self.assertEqual(dep_getusedeps(
								cpv), use)
						else:
							if len(use):
								self.assertEqual(dep_getusedeps(
									cpv + "[" + use + "]"), (use,))
							else:
								self.assertEqual(dep_getusedeps(
									cpv + "[" + use + "]"), ())



#-*- coding:utf-8 -*-
# Copyright 2014-2015 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2
"""
Methods to check whether Portage is going to write to read-only filesystems.
Since the methods are not portable across different OSes, each OS needs its
own method. To expand RO checking for different OSes, add a method which
accepts a list of directories and returns a list of mounts which need to be
remounted RW, then add "elif ostype == (the ostype value for your OS)" to
get_ro_checker().
"""
import io
import logging
import os

from portage import _encodings
from portage.util import writemsg_level
from portage.localization import _
from portage.data import ostype


def get_ro_checker():
	"""
	Uses the system type to find an appropriate method for testing whether Portage
	is going to write to any read-only filesystems.

	@return:
	1. A method for testing for RO filesystems appropriate to the current system.
	"""
	return _CHECKERS.get(ostype, empty_ro_checker)


def linux_ro_checker(dir_list):
	"""
	Use /proc/self/mountinfo to check that no directories installed by the
	ebuild are set to be installed to a read-only filesystem.

	@param dir_list: A list of directories installed by the ebuild.
	@type dir_list: List
	@return:
	1. A list of filesystems which are both set to be written to and are mounted
	read-only, may be empty.
	"""
	ro_filesystems = set()
	invalids = []

	try:
		with io.open("/proc/self/mountinfo", mode='r',
			encoding=_encodings['content'], errors='replace') as f:
			for line in f:
				# we're interested in dir and both attr fileds which always
				# start with either 'ro' or 'rw'
				# example line:
				# 14 1 8:3 / / rw,noatime - ext3 /dev/root rw,errors=continue,commit=5,barrier=1,data=writeback
				#       _dir ^ ^ attr1                     ^ attr2
				# there can be a variable number of fields
				# to the left of the ' - ', after the attr's, so split it there
				mount = line.split(' - ', 1)
				try:
					_dir, attr1 = mount[0].split()[4:6]
				except ValueError:
					# If it raises ValueError we can simply ignore the line.
					invalids.append(line)
					continue
				# check for situation with invalid entries for /home and /root in /proc/self/mountinfo
				# root path is missing sometimes on WSL
				# for example: 16 1 0:16 / /root rw,noatime - lxfs  rw
				if len(mount) > 1:
					try:
						attr2 = mount[1].split()[2]
					except IndexError:
						try:
							attr2 = mount[1].split()[1]
						except IndexError:
							invalids.append(line)
							continue
				else:
					invalids.append(line)
					continue
				if attr1.startswith('ro') or attr2.startswith('ro'):
					ro_filesystems.add(_dir)

	# If /proc/self/mountinfo can't be read, assume that there are no RO
	# filesystems and return.
	except EnvironmentError:
		writemsg_level(_("!!! /proc/self/mountinfo cannot be read"),
			level=logging.WARNING, noiselevel=-1)
		return []

	for line in invalids:
		writemsg_level(_("!!! /proc/self/mountinfo contains unrecognized line: %s\n")
			% line.rstrip(), level=logging.WARNING, noiselevel=-1)

	ro_devs = {}
	for x in ro_filesystems:
		try:
			ro_devs[os.stat(x).st_dev] = x
		except OSError:
			pass

	ro_filesystems.clear()
	for x in set(dir_list):
		try:
			dev = os.stat(x).st_dev
		except OSError:
			pass
		else:
			try:
				ro_filesystems.add(ro_devs[dev])
			except KeyError:
				pass

	return ro_filesystems


def empty_ro_checker(dir_list):
	"""
	Always returns [], this is the fallback function if the system does not have
	an ro_checker method defined.
	"""
	return []


# _CHECKERS is a map from ostype output to the appropriate function to return
# in get_ro_checker.
_CHECKERS = {
	"Linux": linux_ro_checker,
}



# Copyright 2007-2020 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

import subprocess

from portage import os
from portage import _unicode_decode
from portage._sets.base import PackageSet
from portage._sets import SetConfigError

__all__ = ["CommandOutputSet"]

class CommandOutputSet(PackageSet):
	"""This class creates a PackageSet from the output of a shell command.
	   The shell command should produce one atom per line, that is:

	   >>> atom1
	       atom2
	       ...
	       atomN

	   Args:
	     name: A string that identifies the set.
	     command: A string or sequence identifying the command to run
	     (see the subprocess.Popen documentaion for the format)
	"""
	_operations = ["merge", "unmerge"]

	def __init__(self, command):
		super(CommandOutputSet, self).__init__()
		self._command = command
		self.description = "Package set generated from output of '%s'" % self._command

	def load(self):
		pipe = subprocess.Popen(self._command, stdout=subprocess.PIPE, shell=True)
		stdout, stderr = pipe.communicate()
		if pipe.wait() == os.EX_OK:
			self._setAtoms(_unicode_decode(stdout).splitlines())

	def singleBuilder(self, options, settings, trees):
		if not "command" in options:
			raise SetConfigError("no command specified")
		return CommandOutputSet(options["command"])
	singleBuilder = classmethod(singleBuilder)



# Copyright 2013 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

from portage.tests import TestCase
from portage.tests.resolver.ResolverPlayground import (ResolverPlayground,
	ResolverPlaygroundTestCase)

class SlotConflictUpdateTestCase(TestCase):

	def testSlotConflictUpdate(self):

		ebuilds = {

			"app-text/podofo-0.9.2" : {
				"EAPI": "5",
				"RDEPEND" : "dev-util/boost-build"
			},

			"dev-cpp/libcmis-0.3.1" : {
				"EAPI": "5",
				"RDEPEND" : "dev-libs/boost:="
			},

			"dev-libs/boost-1.53.0" : {
				"EAPI": "5",
				"SLOT": "0/1.53",
				"RDEPEND" : "=dev-util/boost-build-1.53.0"
			},

			"dev-libs/boost-1.52.0" : {
				"EAPI": "5",
				"SLOT": "0/1.52",
				"RDEPEND" : "=dev-util/boost-build-1.52.0"
			},

			"dev-util/boost-build-1.53.0" : {
				"EAPI": "5",
				"SLOT": "0"
			},

			"dev-util/boost-build-1.52.0" : {
				"EAPI": "5",
				"SLOT": "0"
			},


		}

		installed = {

			"app-text/podofo-0.9.2" : {
				"EAPI": "5",
				"RDEPEND" : "dev-util/boost-build"
			},

			"dev-cpp/libcmis-0.3.1" : {
				"EAPI": "5",
				"RDEPEND" : "dev-libs/boost:0/1.52="
			},

			"dev-util/boost-build-1.52.0" : {
				"EAPI": "5",
				"SLOT": "0"
			},

			"dev-libs/boost-1.52.0" : {
				"EAPI": "5",
				"SLOT": "0/1.52",
				"RDEPEND" : "=dev-util/boost-build-1.52.0"
			}

		}

		world = ["dev-cpp/libcmis", "dev-libs/boost", "app-text/podofo"]

		test_cases = (

			# In order to avoid a missed update, first mask lower
			# versions that conflict with higher versions. Note that
			# this behavior makes SlotConflictMaskUpdateTestCase
			# fail.
			ResolverPlaygroundTestCase(
				['@world'],
				all_permutations = True,
				options = {"--update": True, "--deep": True},
				success = True,
				mergelist = ['dev-util/boost-build-1.53.0', 'dev-libs/boost-1.53.0', 'dev-cpp/libcmis-0.3.1']),

		)

		playground = ResolverPlayground(ebuilds=ebuilds,
			installed=installed, world=world, debug=False)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True, test_case.fail_msg)
		finally:
			playground.cleanup()



#!/usr/bin/python -b
# Copyright 2015 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

import os
import sys

from portage.util import apply_recursive_permissions

# Change back to original cwd _after_ all imports (bug #469338).
os.chdir(os.environ["__PORTAGE_HELPER_CWD"])

def main(files):

	# We can't trust that the filesystem encoding (locale dependent)
	# correctly matches the arguments, so use surrogateescape to
	# pass through the original argv bytes for Python 3.
	fs_encoding = sys.getfilesystemencoding()
	files = [x.encode(fs_encoding, 'surrogateescape') for x in files]

	for filename in files:
		# Emulate 'chmod -fR a+rX,u+w,g-w,o-w' with minimal chmod calls.
		apply_recursive_permissions(filename, filemode=0o644,
			filemask=0o022, dirmode=0o755, dirmask=0o022)

	return os.EX_OK

if __name__ == "__main__":
	sys.exit(main(sys.argv[1:]))



# Copyright 2010-2013 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

__all__ = ['cacheddir', 'listdir']

import errno
import stat


from portage import os
from portage.const import VCS_DIRS
from portage.exception import DirectoryNotFound, PermissionDenied, PortageException
from portage.util import normalize_path

# The global dircache is no longer supported, since it could
# be a memory leak for API consumers. Any cacheddir callers
# should use higher-level caches instead, when necessary.
# TODO: Remove dircache variable after stable portage does
# not use is (keep it for now, in case API consumers clear
# it manually).
dircache = {}

def cacheddir(my_original_path, ignorecvs, ignorelist, EmptyOnError, followSymlinks=True):
	mypath = normalize_path(my_original_path)
	try:
		pathstat = os.stat(mypath)
		if not stat.S_ISDIR(pathstat.st_mode):
			raise DirectoryNotFound(mypath)
	except EnvironmentError as e:
		if e.errno == PermissionDenied.errno:
			raise PermissionDenied(mypath)
		del e
		return [], []
	except PortageException:
		return [], []
	else:
		try:
			fpaths = os.listdir(mypath)
		except EnvironmentError as e:
			if e.errno != errno.EACCES:
				raise
			del e
			raise PermissionDenied(mypath)
		ftype = []
		for x in fpaths:
			try:
				if followSymlinks:
					pathstat = os.stat(mypath+"/"+x)
				else:
					pathstat = os.lstat(mypath+"/"+x)

				if stat.S_ISREG(pathstat[stat.ST_MODE]):
					ftype.append(0)
				elif stat.S_ISDIR(pathstat[stat.ST_MODE]):
					ftype.append(1)
				elif stat.S_ISLNK(pathstat[stat.ST_MODE]):
					ftype.append(2)
				else:
					ftype.append(3)
			except (IOError, OSError):
				ftype.append(3)

	if ignorelist or ignorecvs:
		ret_list = []
		ret_ftype = []
		for file_path, file_type in zip(fpaths, ftype):
			if file_path in ignorelist:
				pass
			elif ignorecvs:
				if file_path[:2] != ".#" and \
					not (file_type == 1 and file_path in VCS_DIRS):
					ret_list.append(file_path)
					ret_ftype.append(file_type)
	else:
		ret_list = fpaths
		ret_ftype = ftype

	return ret_list, ret_ftype

def listdir(mypath, recursive=False, filesonly=False, ignorecvs=False, ignorelist=[], followSymlinks=True,
	EmptyOnError=False, dirsonly=False):
	"""
	Portage-specific implementation of os.listdir

	@param mypath: Path whose contents you wish to list
	@type mypath: String
	@param recursive: Recursively scan directories contained within mypath
	@type recursive: Boolean
	@param filesonly; Only return files, not more directories
	@type filesonly: Boolean
	@param ignorecvs: Ignore VCS directories
	@type ignorecvs: Boolean
	@param ignorelist: List of filenames/directories to exclude
	@type ignorelist: List
	@param followSymlinks: Follow Symlink'd files and directories
	@type followSymlinks: Boolean
	@param EmptyOnError: Return [] if an error occurs (deprecated, always True)
	@type EmptyOnError: Boolean
	@param dirsonly: Only return directories.
	@type dirsonly: Boolean
	@rtype: List
	@return: A list of files and directories (or just files or just directories) or an empty list.
	"""

	fpaths, ftype = cacheddir(mypath, ignorecvs, ignorelist, EmptyOnError, followSymlinks)

	if fpaths is None:
		fpaths = []
	if ftype is None:
		ftype = []

	if not (filesonly or dirsonly or recursive):
		return fpaths

	if recursive:
		stack = list(zip(fpaths, ftype))
		fpaths = []
		ftype = []
		while stack:
			file_path, file_type = stack.pop()
			fpaths.append(file_path)
			ftype.append(file_type)
			if file_type == 1:
				subdir_list, subdir_types = cacheddir(
					os.path.join(mypath, file_path), ignorecvs,
					ignorelist, EmptyOnError, followSymlinks)
				stack.extend((os.path.join(file_path, x), x_type)
					for x, x_type in zip(subdir_list, subdir_types))

	if filesonly:
		fpaths = [x for x, x_type in zip(fpaths, ftype) if x_type == 0]

	elif dirsonly:
		fpaths = [x for x, x_type in zip(fpaths, ftype) if x_type == 1]

	return fpaths



# Copyright 2015 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2
#
# This is a minimalistic derivation of Python's deprecated formatter module,
# supporting only the methods related to style, literal data, and line breaks.

import sys


class AbstractFormatter:
	"""The standard formatter."""

	def __init__(self, writer):
		self.writer = writer            # Output device
		self.style_stack = []           # Other state, e.g. color
		self.hard_break = True          # Have a hard break

	def add_line_break(self):
		if not self.hard_break:
			self.writer.send_line_break()
		self.hard_break = True

	def add_literal_data(self, data):
		if not data: return
		self.hard_break = data[-1:] == '\n'
		self.writer.send_literal_data(data)

	def push_style(self, *styles):
		for style in styles:
			self.style_stack.append(style)
		self.writer.new_styles(tuple(self.style_stack))

	def pop_style(self, n=1):
		del self.style_stack[-n:]
		self.writer.new_styles(tuple(self.style_stack))


class NullWriter:
	"""Minimal writer interface to use in testing & inheritance.

	A writer which only provides the interface definition; no actions are
	taken on any methods.  This should be the base class for all writers
	which do not need to inherit any implementation methods.
	"""
	def __init__(self): pass
	def flush(self): pass
	def new_styles(self, styles): pass
	def send_line_break(self): pass
	def send_literal_data(self, data): pass


class DumbWriter(NullWriter):
	"""Simple writer class which writes output on the file object passed in
	as the file parameter or, if file is omitted, on standard output.
	"""

	def __init__(self, file=None, maxcol=None):
		NullWriter.__init__(self)
		self.file = file or sys.stdout

	def flush(self):
		self.file.flush()

	def send_line_break(self):
		self.file.write('\n')

	def send_literal_data(self, data):
		self.file.write(data)



# Copyright 2015-2020 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

from portage.exception import InvalidData
from portage.localization import _

class NeededEntry:
	"""
	Represents one entry (line) from a NEEDED.ELF.2 file. The entry
	must have 5 or more semicolon-delimited fields in order to be
	considered valid. The sixth field is optional, corresponding
	to the multilib category. The multilib_category attribute is
	None if the corresponding field is either empty or missing.
	"""

	__slots__ = ("arch", "filename", "multilib_category", "needed",
		"runpaths", "soname")

	_MIN_FIELDS = 5
	_MULTILIB_CAT_INDEX = 5

	@classmethod
	def parse(cls, filename, line):
		"""
		Parse a NEEDED.ELF.2 entry. Raises InvalidData if necessary.

		@param filename: file name for use in exception messages
		@type filename: str
		@param line: a single line of text from a NEEDED.ELF.2 file,
			without a trailing newline
		@type line: str
		@rtype: NeededEntry
		@return: A new NeededEntry instance containing data from line
		"""
		fields = line.split(";")
		if len(fields) < cls._MIN_FIELDS:
			raise InvalidData(_("Wrong number of fields "
				"in %s: %s\n\n") % (filename, line))

		obj = cls()
		# Extra fields may exist (for future extensions).
		if (len(fields) > cls._MULTILIB_CAT_INDEX and
			fields[cls._MULTILIB_CAT_INDEX]):
			obj.multilib_category = fields[cls._MULTILIB_CAT_INDEX]
		else:
			obj.multilib_category = None

		del fields[cls._MIN_FIELDS:]
		obj.arch, obj.filename, obj.soname, rpaths, needed = fields
		# We don't use scanelf -q, since that would omit libraries like
		# musl's /usr/lib/libc.so which do not have any DT_NEEDED or
		# DT_SONAME settings. Since we don't use scanelf -q, we have to
		# handle the special rpath value "  -  " below.
		rpaths = "" if rpaths == "  -  " else rpaths
		obj.runpaths = tuple(filter(None, rpaths.split(":")))
		obj.needed = tuple(filter(None, needed.split(",")))

		return obj

	def __str__(self):
		"""
		Format this entry for writing to a NEEDED.ELF.2 file.
		"""
		return ";".join([
				self.arch,
				self.filename,
				self.soname,
				":".join(self.runpaths),
				",".join(self.needed),
				(self.multilib_category if self.multilib_category
				is not None else "")
		]) + "\n"



# Copyright 2013 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

from portage import os
from portage.checksum import perform_multiple_checksums
from portage.util._async.ForkProcess import ForkProcess
from _emerge.PipeReader import PipeReader

class FileDigester(ForkProcess):
	"""
	Asynchronously generate file digests. Pass in file_path and
	hash_names, and after successful execution, the digests
	attribute will be a dict containing all of the requested
	digests.
	"""

	__slots__ = ('file_path', 'digests', 'hash_names',
		'_digest_pipe_reader', '_digest_pw')

	def _start(self):
		pr, pw = os.pipe()
		self.fd_pipes = {}
		self.fd_pipes[pw] = pw
		self._digest_pw = pw
		self._digest_pipe_reader = PipeReader(
			input_files={"input":pr},
			scheduler=self.scheduler)
		self._digest_pipe_reader.addExitListener(self._digest_pipe_reader_exit)
		self._digest_pipe_reader.start()
		ForkProcess._start(self)
		os.close(pw)

	def _run(self):
		digests = perform_multiple_checksums(self.file_path,
			hashes=self.hash_names)

		buf = "".join("%s=%s\n" % item
			for item in digests.items()).encode('utf_8')

		while buf:
			buf = buf[os.write(self._digest_pw, buf):]

		return os.EX_OK

	def _parse_digests(self, data):

		digests = {}
		for line in data.decode('utf_8').splitlines():
			parts = line.split('=', 1)
			if len(parts) == 2:
				digests[parts[0]] = parts[1]

		self.digests = digests

	def _async_waitpid(self):
		# Ignore this event, since we want to ensure that we
		# exit only after _digest_pipe_reader has reached EOF.
		if self._digest_pipe_reader is None:
			ForkProcess._async_waitpid(self)

	def _digest_pipe_reader_exit(self, pipe_reader):
		self._parse_digests(pipe_reader.getvalue())
		self._digest_pipe_reader = None
		if self.pid is None:
			self._unregister()
			self._async_wait()
		else:
			self._async_waitpid()

	def _unregister(self):
		ForkProcess._unregister(self)

		pipe_reader = self._digest_pipe_reader
		if pipe_reader is not None:
			self._digest_pipe_reader = None
			pipe_reader.removeExitListener(self._digest_pipe_reader_exit)
			pipe_reader.cancel()



# Copyright 2014 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

import collections

from portage.dep import Atom
from portage.tests import TestCase
from _emerge.resolver.package_tracker import PackageTracker, PackageTrackerDbapiWrapper

class PackageTrackerTestCase(TestCase):

	FakePackage = collections.namedtuple("FakePackage",
		["root", "cp", "cpv", "slot", "slot_atom", "version", "repo"])

	FakeConflict = collections.namedtuple("FakeConflict",
		["description", "root", "pkgs"])

	def make_pkg(self, root, atom, repo="test_repo"):
		atom = Atom(atom)
		slot_atom = Atom("%s:%s" % (atom.cp, atom.slot))
		slot = atom.slot

		return self.FakePackage(root=root, cp=atom.cp, cpv=atom.cpv,
			slot=slot, slot_atom=slot_atom, version=atom.version, repo=repo)

	def make_conflict(self, description, root, pkgs):
		return self.FakeConflict(description=description, root=root, pkgs=pkgs)

	def test_add_remove_discard(self):
		p = PackageTracker()

		x1 = self.make_pkg("/", "=dev-libs/X-1:0")
		x2 = self.make_pkg("/", "=dev-libs/X-2:0")

		p.add_pkg(x1)
		self.assertTrue(x1 in p)
		self.assertTrue(p.contains(x1, installed=True))
		self.assertTrue(p.contains(x1, installed=False))
		p.remove_pkg(x1)
		self.assertTrue(x1 not in p)

		p.add_pkg(x1)
		self.assertTrue(x1 in p)
		p.add_pkg(x1)
		self.assertTrue(x1 in p)

		self.assertRaises(KeyError, p.remove_pkg, x2)

		p.add_pkg(x2)
		self.assertTrue(x2 in p)
		p.remove_pkg(x2)
		self.assertTrue(x2 not in p)
		p.discard_pkg(x2)
		self.assertTrue(x2 not in p)
		p.add_pkg(x2)
		self.assertTrue(x2 in p)

		all_pkgs = list(p.all_pkgs("/"))
		self.assertEqual(len(all_pkgs), 2)
		self.assertTrue(all_pkgs[0] is x1 and all_pkgs[1] is x2)

		self.assertEqual(len(list(p.all_pkgs("/"))), 2)
		self.assertEqual(len(list(p.all_pkgs("/xxx"))), 0)

	def test_match(self):
		p = PackageTracker()
		x1 = self.make_pkg("/", "=dev-libs/X-1:0")
		x2 = self.make_pkg("/", "=dev-libs/X-2:0")
		x3 = self.make_pkg("/", "=dev-libs/X-3:1")

		p.add_pkg(x2)
		p.add_pkg(x1)

		matches = list(p.match("/", Atom("=dev-libs/X-1")))
		self.assertTrue(x1 in matches)
		self.assertEqual(len(matches), 1)

		matches = list(p.match("/", Atom("dev-libs/X")))
		self.assertTrue(x1 is matches[0] and x2 is matches[1])
		self.assertEqual(len(matches), 2)

		matches = list(p.match("/xxx", Atom("dev-libs/X")))
		self.assertEqual(len(matches), 0)

		matches = list(p.match("/", Atom("dev-libs/Y")))
		self.assertEqual(len(matches), 0)

		p.add_pkg(x3)
		matches = list(p.match("/", Atom("dev-libs/X")))
		self.assertTrue(x1 is matches[0] and x2 is matches[1] and x3 is matches[2])
		self.assertEqual(len(matches), 3)

		p.remove_pkg(x3)
		matches = list(p.match("/", Atom("dev-libs/X")))
		self.assertTrue(x1 is matches[0] and x2 is matches[1])
		self.assertEqual(len(matches), 2)

	def test_dbapi_interface(self):
		p = PackageTracker()
		dbapi = PackageTrackerDbapiWrapper("/", p)
		installed = self.make_pkg("/", "=dev-libs/X-0:0")
		x1 = self.make_pkg("/", "=dev-libs/X-1:0")
		x2 = self.make_pkg("/", "=dev-libs/X-2:0")
		x3 = self.make_pkg("/", "=dev-libs/X-3:0")
		x4 = self.make_pkg("/", "=dev-libs/X-4:6")
		x5 = self.make_pkg("/xxx", "=dev-libs/X-5:6")

		def check_dbapi(pkgs):
			all_pkgs = set(dbapi)
			self.assertEqual(len(all_pkgs), len(pkgs))

			x_atom = "dev-libs/X"
			y_atom = "dev-libs/Y"
			matches = dbapi.cp_list(x_atom)
			for pkg in pkgs:
				if pkg.root == "/" and pkg.cp == x_atom:
					self.assertTrue(pkg in matches)
			self.assertTrue(not dbapi.cp_list(y_atom))
			matches = dbapi.match(Atom(x_atom))
			for pkg in pkgs:
				if pkg.root == "/" and pkg.cp == x_atom:
					self.assertTrue(pkg in matches)
			self.assertTrue(not dbapi.match(Atom(y_atom)))

		check_dbapi([])

		p.add_installed_pkg(installed)
		check_dbapi([installed])

		p.add_pkg(x1)
		check_dbapi([x1])

		p.remove_pkg(x1)
		check_dbapi([installed])

		dbapi.cpv_inject(x1)
		check_dbapi([x1])

		dbapi.cpv_inject(x2)
		check_dbapi([x1, x2])

		p.remove_pkg(x1)
		check_dbapi([x2])

		p.add_pkg(x5)
		check_dbapi([x2])


	def test_installed(self):
		p = PackageTracker()
		x1 = self.make_pkg("/", "=dev-libs/X-1:0")
		x1b = self.make_pkg("/", "=dev-libs/X-1.1:0")
		x2 = self.make_pkg("/", "=dev-libs/X-2:0")
		x3 = self.make_pkg("/", "=dev-libs/X-3:1")

		def check_installed(x, should_contain, num_pkgs):
			self.assertEqual(x in p, should_contain)
			self.assertEqual(p.contains(x), should_contain)
			self.assertEqual(p.contains(x1, installed=True), should_contain)
			self.assertEqual(p.contains(x1, installed=False), False)
			self.assertEqual(len(list(p.all_pkgs("/"))), num_pkgs)

		def check_matches(atom, expected):
			matches = list(p.match("/", Atom(atom)))
			self.assertEqual(len(matches), len(expected))
			for x, y in zip(matches, expected):
				self.assertTrue(x is y)

		p.add_installed_pkg(x1)
		check_installed(x1, True, 1)
		check_matches("dev-libs/X", [x1])

		p.add_installed_pkg(x1)
		check_installed(x1, True, 1)
		check_matches("dev-libs/X", [x1])

		p.add_pkg(x2)
		check_installed(x1, False, 1)
		check_matches("dev-libs/X", [x2])

		p.add_installed_pkg(x1)
		check_installed(x1, False, 1)
		check_matches("dev-libs/X", [x2])

		p.add_installed_pkg(x1b)
		check_installed(x1, False, 1)
		check_installed(x1b, False, 1)
		check_matches("dev-libs/X", [x2])

		p.remove_pkg(x2)
		check_installed(x1, True, 2)
		check_installed(x1b, True, 2)
		check_matches("dev-libs/X", [x1, x1b])

	def test_conflicts(self):
		p = PackageTracker()
		installed1 = self.make_pkg("/", "=dev-libs/X-0:0")
		installed2 = self.make_pkg("/", "=dev-libs/X-0.1:0")
		x1 = self.make_pkg("/", "=dev-libs/X-1:0")
		x2 = self.make_pkg("/", "=dev-libs/X-2:0")
		x3 = self.make_pkg("/", "=dev-libs/X-3:0")
		x4 = self.make_pkg("/", "=dev-libs/X-4:4")
		x4b = self.make_pkg("/", "=dev-libs/X-4:4b::x-repo")

		def check_conflicts(expected, slot_conflicts_only=False):
			if slot_conflicts_only:
				conflicts = list(p.slot_conflicts())
			else:
				conflicts = list(p.conflicts())
			self.assertEqual(len(conflicts), len(expected))
			for got, exp in zip(conflicts, expected):
				self.assertEqual(got.description, exp.description)
				self.assertEqual(got.root, exp.root)
				self.assertEqual(len(got.pkgs), len(exp.pkgs))
				self.assertEqual(len(got), len(exp.pkgs))
				for x, y in zip(got.pkgs, exp.pkgs):
					self.assertTrue(x is y)
				for x, y in zip(got, exp.pkgs):
					self.assertTrue(x is y)
				for x in exp.pkgs:
					self.assertTrue(x in got)

		check_conflicts([])
		check_conflicts([])

		p.add_installed_pkg(installed1)
		p.add_installed_pkg(installed2)
		check_conflicts([])

		p.add_pkg(x1)
		check_conflicts([])
		p.add_pkg(x2)
		check_conflicts([self.make_conflict("slot conflict", "/", [x1, x2])])
		p.add_pkg(x3)
		check_conflicts([self.make_conflict("slot conflict", "/", [x1, x2, x3])])
		p.remove_pkg(x3)
		check_conflicts([self.make_conflict("slot conflict", "/", [x1, x2])])
		p.remove_pkg(x2)
		check_conflicts([])
		p.add_pkg(x3)
		check_conflicts([self.make_conflict("slot conflict", "/", [x1, x3])])
		p.add_pkg(x2)
		check_conflicts([self.make_conflict("slot conflict", "/", [x1, x3, x2])])

		p.add_pkg(x4)
		check_conflicts([self.make_conflict("slot conflict", "/", [x1, x3, x2])])

		p.add_pkg(x4b)
		check_conflicts(
			[
			self.make_conflict("slot conflict", "/", [x1, x3, x2]),
			self.make_conflict("cpv conflict", "/", [x4, x4b]),
			]
			)

		check_conflicts(
			[
			self.make_conflict("slot conflict", "/", [x1, x3, x2]),
			],
			slot_conflicts_only=True
			)



# Copyright 2012-2019 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

from portage.tests import TestCase
from portage.tests.resolver.ResolverPlayground import ResolverPlayground, ResolverPlaygroundTestCase

class TargetRootTestCase(TestCase):

	def testTargetRoot(self):
		ebuilds = {
			"dev-lang/python-3.2": {
				"EAPI": "7",
				"BDEPEND": "~dev-lang/python-3.2",
			},
			"dev-libs/A-1": {
				"EAPI": "4",
				"DEPEND": "dev-libs/B",
				"RDEPEND": "dev-libs/C",
			},
			"dev-libs/B-1": {},
			"dev-libs/C-1": {},
		}

		installed = {
			"dev-lang/python-3.2": {
				"EAPI": "7",
				"BDEPEND": "~dev-lang/python-3.2",
			},
		}

		test_cases = (
			ResolverPlaygroundTestCase(
				["dev-lang/python"],
				options = {},
				success = True,
				mergelist = ["dev-lang/python-3.2{targetroot}"]),
			ResolverPlaygroundTestCase(
				["dev-lang/python"],
				options = {"--root-deps": True},
				success = True,
				mergelist = ["dev-lang/python-3.2{targetroot}"]),
			ResolverPlaygroundTestCase(
				["dev-lang/python"],
				options = {"--root-deps": "rdeps"},
				success = True,
				mergelist = ["dev-lang/python-3.2{targetroot}"]),
			ResolverPlaygroundTestCase(
				["dev-libs/A"],
				options = {},
				ambiguous_merge_order = True,
				success = True,
				mergelist = [("dev-libs/B-1", "dev-libs/C-1{targetroot}"), "dev-libs/A-1{targetroot}"]),
			ResolverPlaygroundTestCase(
				["dev-libs/A"],
				options = {"--root-deps": True},
				ambiguous_merge_order = True,
				success = True,
				mergelist = [("dev-libs/B-1{targetroot}", "dev-libs/C-1{targetroot}"), "dev-libs/A-1{targetroot}"]),
			ResolverPlaygroundTestCase(
				["dev-libs/A"],
				options = {"--root-deps": "rdeps"},
				ambiguous_merge_order = True,
				success = True,
				mergelist = [("dev-libs/C-1{targetroot}"), "dev-libs/A-1{targetroot}"]),
		)

		playground = ResolverPlayground(ebuilds=ebuilds, installed=installed, targetroot=True,
			debug=False)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True, test_case.fail_msg)
		finally:
			playground.cleanup()

		test_cases = (
			ResolverPlaygroundTestCase(
				["dev-lang/python"],
				options = {},
				success = True,
				mergelist = ["dev-lang/python-3.2"]),
		)

		playground = ResolverPlayground(ebuilds=ebuilds, installed=installed, targetroot=False,
			debug=False)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True, test_case.fail_msg)
		finally:
			playground.cleanup()



# Copyright 2014 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

from portage.tests import TestCase
from portage.tests.resolver.ResolverPlayground import (
	ResolverPlayground, ResolverPlaygroundTestCase)

class ChangedDepsTestCase(TestCase):

	def testChangedDeps(self):

		ebuilds = {
			"app-misc/A-0": {
				"DEPEND": "app-misc/B",
				"RDEPEND": "app-misc/B",
			},
			"app-misc/B-0": {
			}
		}

		binpkgs = {
			"app-misc/A-0": {},
		}

		installed = {
			"app-misc/A-0": {},
		}

		world= (
			"app-misc/A",
		)

		test_cases = (

			# --dynamic-deps=n causes the original deps to be respected
			ResolverPlaygroundTestCase(
				["@world"],
				success = True,
				options = {
					"--update": True,
					"--deep": True,
					"--dynamic-deps": "n",
					"--usepkg": True,
				},
				mergelist = []
			),

			# --dynamic-deps causes app-misc/B to get pulled in
			ResolverPlaygroundTestCase(
				["@world"],
				success = True,
				options = {
					"--update": True,
					"--deep": True,
					"--dynamic-deps": "y",
					"--usepkg": True,
				},
				mergelist = ["app-misc/B-0"]
			),

			# --changed-deps causes app-misc/A to be rebuilt
			ResolverPlaygroundTestCase(
				["@world"],
				success = True,
				options = {
					"--update": True,
					"--deep": True,
					"--changed-deps": "y",
					"--usepkg": True,
				},
				mergelist = ["app-misc/B-0", "app-misc/A-0"]
			),

			# --usepkgonly prevents automatic --binpkg-changed-deps
			ResolverPlaygroundTestCase(
				["app-misc/A"],
				success = True,
				options = {
					"--changed-deps": "y",
					"--usepkgonly": True,
				},
				mergelist = ["[binary]app-misc/A-0"]
			),

			# Test automatic --binpkg-changed-deps, which cases the
			# binpkg with stale deps to be ignored (with warning
			# message)
			ResolverPlaygroundTestCase(
				["app-misc/A"],
				success = True,
				options = {
					"--usepkg": True,
				},
				mergelist = ["app-misc/B-0", "app-misc/A-0"]
			),
		)
		test_cases = (

			# Forcibly disable --binpkg-changed-deps, which causes
			# --changed-deps to be overridden by --binpkg-changed-deps
			ResolverPlaygroundTestCase(
				["app-misc/A"],
				success = True,
				options = {
					"--binpkg-changed-deps": "n",
					"--changed-deps": "y",
					"--usepkg": True,
				},
				mergelist = ["[binary]app-misc/A-0"]
			),
		)

		playground = ResolverPlayground(debug=False, ebuilds=ebuilds,
			binpkgs=binpkgs, installed=installed, world=world)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success,
					True, test_case.fail_msg)
		finally:
			playground.cleanup()



# Copyright 2018-2021 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

import os
import signal

import asyncio as _real_asyncio
from asyncio.events import AbstractEventLoop as _AbstractEventLoop
from asyncio.unix_events import AbstractChildWatcher as _AbstractChildWatcher

import portage


class AsyncioEventLoop(_AbstractEventLoop):
	"""
	Implementation of asyncio.AbstractEventLoop which wraps asyncio's
	event loop and is minimally compatible with _PortageEventLoop.
	"""

	def __init__(self, loop=None):
		loop = loop or _real_asyncio.get_event_loop()
		self._loop = loop
		self.run_until_complete = (self._run_until_complete
			if portage._internal_caller else loop.run_until_complete)
		self.call_soon = loop.call_soon
		self.call_soon_threadsafe = loop.call_soon_threadsafe
		self.call_later = loop.call_later
		self.call_at = loop.call_at
		self.is_running = loop.is_running
		self.is_closed = loop.is_closed
		self.close = loop.close
		self.create_future = (loop.create_future
			if hasattr(loop, 'create_future') else self._create_future)
		self.create_task = loop.create_task
		self.add_reader = loop.add_reader
		self.remove_reader = loop.remove_reader
		self.add_writer = loop.add_writer
		self.remove_writer = loop.remove_writer
		self.run_in_executor = loop.run_in_executor
		self.time = loop.time
		self.default_exception_handler = loop.default_exception_handler
		self.call_exception_handler = loop.call_exception_handler
		self.set_debug = loop.set_debug
		self.get_debug = loop.get_debug
		self._wakeup_fd = -1
		self._child_watcher = None

		if portage._internal_caller:
			loop.set_exception_handler(self._internal_caller_exception_handler)

	@staticmethod
	def _internal_caller_exception_handler(loop, context):
		"""
		An exception handler which drops to a pdb shell if std* streams
		refer to a tty, and otherwise kills the process with SIGTERM.

		In order to avoid potential interference with API consumers, this
		implementation is only used when portage._internal_caller is True.
		"""
		loop.default_exception_handler(context)
		if 'exception' in context:
			# Normally emerge will wait for all coroutines to complete
			# after SIGTERM has been received. However, an unhandled
			# exception will prevent the interrupted coroutine from
			# completing, therefore use the default SIGTERM handler
			# in order to ensure that emerge exits immediately (though
			# uncleanly).
			signal.signal(signal.SIGTERM, signal.SIG_DFL)
			os.kill(portage.getpid(), signal.SIGTERM)

	def _create_future(self):
		"""
		Provide AbstractEventLoop.create_future() for python3.4.
		"""
		return _real_asyncio.Future(loop=self._loop)

	@property
	def _asyncio_child_watcher(self):
		"""
		Portage internals use this as a layer of indirection for
		asyncio.get_child_watcher(), in order to support versions of
		python where asyncio is not available.

		@rtype: asyncio.AbstractChildWatcher
		@return: the internal event loop's AbstractChildWatcher interface
		"""
		if self._child_watcher is None:
			self._child_watcher = _ChildWatcherThreadSafetyWrapper(self, _real_asyncio.get_child_watcher())
		return self._child_watcher

	@property
	def _asyncio_wrapper(self):
		"""
		Portage internals use this as a layer of indirection in cases
		where a wrapper around an asyncio.AbstractEventLoop implementation
		is needed for purposes of compatiblity.

		@rtype: asyncio.AbstractEventLoop
		@return: the internal event loop's AbstractEventLoop interface
		"""
		return self

	def _run_until_complete(self, future):
		"""
		An implementation of AbstractEventLoop.run_until_complete that supresses
		spurious error messages like the following reported in bug 655656:

		    Exception ignored when trying to write to the signal wakeup fd:
		    BlockingIOError: [Errno 11] Resource temporarily unavailable

		In order to avoid potential interference with API consumers, this
		implementation is only used when portage._internal_caller is True.
		"""
		if self._wakeup_fd != -1:
			signal.set_wakeup_fd(self._wakeup_fd)
			self._wakeup_fd = -1
			# Account for any signals that may have arrived between
			# set_wakeup_fd calls.
			os.kill(portage.getpid(), signal.SIGCHLD)
		try:
			return self._loop.run_until_complete(future)
		finally:
			try:
				self._wakeup_fd = signal.set_wakeup_fd(-1)
			except ValueError:
				# This is intended to fail when not called in the main thread.
				pass


class _ChildWatcherThreadSafetyWrapper(_AbstractChildWatcher):
	def __init__(self, loop, real_watcher):
		self._loop = loop
		self._real_watcher = real_watcher

	def close(self):
		pass

	def __enter__(self):
		return self

	def __exit__(self, a, b, c):
		pass

	def _child_exit(self, pid, status, callback, *args):
		self._loop.call_soon_threadsafe(callback, pid, status, *args)

	def add_child_handler(self, pid, callback, *args):
		self._real_watcher.add_child_handler(pid, self._child_exit, callback, *args)

	def remove_child_handler(self, pid):
		return self._real_watcher.remove_child_handler(pid)



# Copyright 2017 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

from portage.tests import TestCase
from portage.tests.resolver.ResolverPlayground import (
	ResolverPlayground,
	ResolverPlaygroundTestCase,
)

class BinaryPkgEbuildVisibilityTestCase(TestCase):

	def testBinaryPkgEbuildVisibility(self):

		binpkgs = {
			"app-misc/foo-3" : {},
			"app-misc/foo-2" : {},
			"app-misc/foo-1" : {},
		}

		ebuilds = {
			"app-misc/foo-2" : {},
			"app-misc/foo-1" : {},
		}

		installed = {
			"app-misc/foo-1" : {},
		}

		world = ["app-misc/foo"]

		test_cases = (

			# Test bug #612960, where --use-ebuild-visibility failed
			# to reject binary packages for which ebuilds were not
			# available.
			ResolverPlaygroundTestCase(
				["@world"],
				options = {
					"--update": True,
					"--deep": True,
					"--use-ebuild-visibility": 'y',
					"--usepkgonly": True,
				},
				success = True,
				mergelist = [
					'[binary]app-misc/foo-2',
				],
			),

			ResolverPlaygroundTestCase(
				["@world"],
				options = {
					"--update": True,
					"--deep": True,
					"--usepkgonly": True,
				},
				success = True,
				mergelist = [
					'[binary]app-misc/foo-3',
				],
			),

			ResolverPlaygroundTestCase(
				["@world"],
				options = {
					"--update": True,
					"--deep": True,
					"--usepkg": True,
				},
				success = True,
				mergelist = [
					'[binary]app-misc/foo-2',
				],
			),

			ResolverPlaygroundTestCase(
				["=app-misc/foo-3"],
				options = {
					"--use-ebuild-visibility": 'y',
					"--usepkgonly": True,
				},
				success = False,
			),

			ResolverPlaygroundTestCase(
				["app-misc/foo"],
				options = {
					"--use-ebuild-visibility": 'y',
					"--usepkgonly": True,
				},
				success = True,
				mergelist = [
					'[binary]app-misc/foo-2',
				],
			),

			ResolverPlaygroundTestCase(
				["app-misc/foo"],
				options = {
					"--usepkgonly": True,
				},
				success = True,
				mergelist = [
					'[binary]app-misc/foo-3',
				],
			),

			# The default behavior is to enforce ebuild visibility as
			# long as a visible package is available to satisfy the
			# current atom. In the following test case, ebuild visibility
			# is ignored in order to satisfy the =app-misc/foo-3 atom.
			ResolverPlaygroundTestCase(
				["=app-misc/foo-3"],
				options = {
					"--usepkg": True,
				},
				success = True,
				mergelist = [
					'[binary]app-misc/foo-3',
				],
			),

			# Verify that --use-ebuild-visibility works with --usepkg
			# when no other visible package is available.
			ResolverPlaygroundTestCase(
				["=app-misc/foo-3"],
				options = {
					"--use-ebuild-visibility": "y",
					"--usepkg": True,
				},
				success = False,
			),
		)

		playground = ResolverPlayground(binpkgs=binpkgs, ebuilds=ebuilds,
			installed=installed, world=world)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True,
					test_case.fail_msg)
		finally:
			playground.cleanup()



# elog/mod_mail_summary.py - elog dispatch module
# Copyright 2006-2020 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

import portage
from portage.exception import AlarmSignal, PortageException
from portage.localization import _
from portage.util import writemsg
from portage import _encodings
from portage import _unicode_decode

import socket
import time

_config_keys = ('PORTAGE_ELOG_MAILURI', 'PORTAGE_ELOG_MAILFROM',
	'PORTAGE_ELOG_MAILSUBJECT',)
_items = {}
def process(mysettings, key, logentries, fulltext):
	global _items
	time_str = _unicode_decode(
		time.strftime("%Y%m%d-%H%M%S %Z", time.localtime(time.time())),
		encoding=_encodings['content'], errors='replace')
	header = _(">>> Messages generated for package %(pkg)s by process %(pid)d on %(time)s:\n\n") % \
		{"pkg": key, "pid": portage.getpid(), "time": time_str}
	config_root = mysettings["PORTAGE_CONFIGROOT"]

	# Copy needed variables from the config instance,
	# since we don't need to hold a reference for the
	# whole thing. This also makes it possible to
	# rely on per-package variable settings that may
	# have come from /etc/portage/package.env, since
	# we'll be isolated from any future mutations of
	# mysettings.
	config_dict = {}
	for k in _config_keys:
		v = mysettings.get(k)
		if v is not None:
			config_dict[k] = v

	config_dict, items = _items.setdefault(config_root, (config_dict, {}))
	items[key] = header + fulltext

def finalize():
	global _items
	for mysettings, items in _items.values():
		_finalize(mysettings, items)
	_items.clear()

def _finalize(mysettings, items):
	if len(items) == 0:
		return
	if len(items) == 1:
		count = _("one package")
	else:
		count = _("multiple packages")
	if "PORTAGE_ELOG_MAILURI" in mysettings:
		myrecipient = mysettings["PORTAGE_ELOG_MAILURI"].split()[0]
	else:
		myrecipient = "root@localhost"

	myfrom = mysettings.get("PORTAGE_ELOG_MAILFROM", "")
	myfrom = myfrom.replace("${HOST}", socket.getfqdn())
	mysubject = mysettings.get("PORTAGE_ELOG_MAILSUBJECT", "")
	mysubject = mysubject.replace("${PACKAGE}", count)
	mysubject = mysubject.replace("${HOST}", socket.getfqdn())

	mybody = _("elog messages for the following packages generated by "
		"process %(pid)d on host %(host)s:\n") % {"pid": portage.getpid(), "host": socket.getfqdn()}
	for key in items:
		mybody += "- %s\n" % key

	mymessage = portage.mail.create_message(myfrom, myrecipient, mysubject,
		mybody, attachments=list(items.values()))

	# Timeout after one minute in case send_mail() blocks indefinitely.
	try:
		try:
			AlarmSignal.register(60)
			portage.mail.send_mail(mysettings, mymessage)
		finally:
			AlarmSignal.unregister()
	except AlarmSignal:
		writemsg("Timeout in finalize() for elog system 'mail_summary'\n",
			noiselevel=-1)
	except PortageException as e:
		writemsg("%s\n" % (e,), noiselevel=-1)

	return



# Copyright 1999-2012, 2016 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

import logging

import portage
from portage import os
from portage.localization import _
from portage.output import bold, colorize, yellow
from portage.util import writemsg_level

def chk_updated_cfg_files(eroot, config_protect):
	target_root = eroot
	result = list(
		portage.util.find_updated_config_files(target_root, config_protect))

	for x in result:
		writemsg_level("\n %s " % (colorize("WARN", "* " + _("IMPORTANT:"))),
			level=logging.INFO, noiselevel=-1)
		if not x[1]: # it's a protected file
			writemsg_level( _("config file '%s' needs updating.\n") % x[0],
				level=logging.INFO, noiselevel=-1)
		else: # it's a protected dir
			if len(x[1]) == 1:
				head, tail = os.path.split(x[1][0])
				tail = tail[len("._cfg0000_"):]
				fpath = os.path.join(head, tail)
				writemsg_level(_("config file '%s' needs updating.\n") % fpath,
					level=logging.INFO, noiselevel=-1)
			else:
				writemsg_level(
					_("%d config files in '%s' need updating.\n") % \
					(len(x[1]), x[0]), level=logging.INFO, noiselevel=-1)

	if result:
		print(" " + yellow("*") + " See the " +
			colorize("INFORM", _("CONFIGURATION FILES")) + " and " +
			colorize("INFORM", _("CONFIGURATION FILES UPDATE TOOLS")))
		print(" " + yellow("*") + " sections of the " + bold("emerge") + " " +
			_("man page to learn how to update config files."))



# Copyright 2018 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

import functools

from portage.util._async.AsyncTaskFuture import AsyncTaskFuture
from portage.util._async.TaskScheduler import TaskScheduler
from portage.util.futures import asyncio
from portage.util.cpuinfo import get_cpu_count


def iter_completed(futures, max_jobs=None, max_load=None, loop=None):
	"""
	This is similar to asyncio.as_completed, but takes an iterator of
	futures as input, and includes support for max_jobs and max_load
	parameters.

	@param futures: iterator of asyncio.Future (or compatible)
	@type futures: iterator
	@param max_jobs: max number of futures to process concurrently (default
		is portage.util.cpuinfo.get_cpu_count())
	@type max_jobs: int
	@param max_load: max load allowed when scheduling a new future,
		otherwise schedule no more than 1 future at a time (default
		is portage.util.cpuinfo.get_cpu_count())
	@type max_load: int or float
	@param loop: event loop
	@type loop: EventLoop
	@return: iterator of futures that are done
	@rtype: iterator
	"""
	loop = asyncio._wrap_loop(loop)

	for future_done_set in async_iter_completed(futures,
		max_jobs=max_jobs, max_load=max_load, loop=loop):
		for future in loop.run_until_complete(future_done_set):
			yield future


def async_iter_completed(futures, max_jobs=None, max_load=None, loop=None):
	"""
	An asynchronous version of iter_completed. This yields futures, which
	when done, result in a set of input futures that are done. This serves
	as a wrapper around portage's internal TaskScheduler class, using
	standard asyncio interfaces.

	@param futures: iterator of asyncio.Future (or compatible)
	@type futures: iterator
	@param max_jobs: max number of futures to process concurrently (default
		is portage.util.cpuinfo.get_cpu_count())
	@type max_jobs: int
	@param max_load: max load allowed when scheduling a new future,
		otherwise schedule no more than 1 future at a time (default
		is portage.util.cpuinfo.get_cpu_count())
	@type max_load: int or float
	@param loop: event loop
	@type loop: EventLoop
	@return: iterator of futures, which when done, result in a set of
		input futures that are done
	@rtype: iterator
	"""
	loop = asyncio._wrap_loop(loop)

	max_jobs = max_jobs or get_cpu_count()
	max_load = max_load or get_cpu_count()

	future_map = {}
	def task_generator():
		for future in futures:
			future_map[id(future)] = future
			yield AsyncTaskFuture(future=future)

	scheduler = TaskScheduler(
		task_generator(),
		max_jobs=max_jobs,
		max_load=max_load,
		event_loop=loop)

	def done_callback(future_done_set, wait_result):
		"""Propagate results from wait_result to future_done_set."""
		if future_done_set.cancelled():
			return
		done, pending = wait_result.result()
		for future in done:
			del future_map[id(future)]
		future_done_set.set_result(done)

	def cancel_callback(wait_result, future_done_set):
		"""Cancel wait_result if future_done_set has been cancelled."""
		if future_done_set.cancelled() and not wait_result.done():
			wait_result.cancel()

	try:
		scheduler.start()

		# scheduler should ensure that future_map is non-empty until
		# task_generator is exhausted
		while future_map:
			wait_result = asyncio.ensure_future(
				asyncio.wait(list(future_map.values()),
				return_when=asyncio.FIRST_COMPLETED, loop=loop), loop=loop)
			future_done_set = loop.create_future()
			future_done_set.add_done_callback(
				functools.partial(cancel_callback, wait_result))
			wait_result.add_done_callback(
				functools.partial(done_callback, future_done_set))
			yield future_done_set
	finally:
		# cleanup in case of interruption by SIGINT, etc
		if not loop.is_closed():
			scheduler.cancel()


def iter_gather(futures, max_jobs=None, max_load=None, loop=None):
	"""
	This is similar to asyncio.gather, but takes an iterator of
	futures as input, and includes support for max_jobs and max_load
	parameters.

	@param futures: iterator of asyncio.Future (or compatible)
	@type futures: iterator
	@param max_jobs: max number of futures to process concurrently (default
		is portage.util.cpuinfo.get_cpu_count())
	@type max_jobs: int
	@param max_load: max load allowed when scheduling a new future,
		otherwise schedule no more than 1 future at a time (default
		is portage.util.cpuinfo.get_cpu_count())
	@type max_load: int or float
	@param loop: event loop
	@type loop: EventLoop
	@return: a Future resulting in a list of done input futures, in the
		same order that they were yielded from the input iterator
	@rtype: asyncio.Future (or compatible)
	"""
	loop = asyncio._wrap_loop(loop)
	result = loop.create_future()
	futures_list = []

	def future_generator():
		for future in futures:
			futures_list.append(future)
			yield future

	completed_iter = async_iter_completed(
		future_generator(),
		max_jobs=max_jobs,
		max_load=max_load,
		loop=loop,
	)

	def handle_result(future_done_set):
		if result.cancelled():
			if not future_done_set.cancelled():
				# All exceptions must be consumed from future_done_set, in order
				# to avoid triggering the event loop's exception handler.
				list(future.exception() for future in future_done_set.result()
					if not future.cancelled())
			return

		try:
			handle_result.current_task = next(completed_iter)
		except StopIteration:
			result.set_result(futures_list)
		else:
			handle_result.current_task.add_done_callback(handle_result)

	try:
		handle_result.current_task = next(completed_iter)
	except StopIteration:
		handle_result.current_task = None
		result.set_result(futures_list)
	else:
		handle_result.current_task.add_done_callback(handle_result)

	def cancel_callback(result):
		if (result.cancelled() and
			handle_result.current_task is not None and
			not handle_result.current_task.done()):
			handle_result.current_task.cancel()

	result.add_done_callback(cancel_callback)

	return result



# Copyright 2012 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

import portage
from portage import os
from portage.exception import (FileNotFound,
	PermissionDenied, PortagePackageException)
from portage.localization import _
from portage.util._async.ForkProcess import ForkProcess

class ManifestProcess(ForkProcess):

	__slots__ = ("cp", "distdir", "fetchlist_dict", "repo_config")

	MODIFIED = 16

	def _run(self):
		mf = self.repo_config.load_manifest(
			os.path.join(self.repo_config.location, self.cp),
			self.distdir, fetchlist_dict=self.fetchlist_dict)

		try:
			mf.create(assumeDistHashesAlways=True)
		except FileNotFound as e:
			portage.writemsg(_("!!! File %s doesn't exist, can't update "
				"Manifest\n") % e, noiselevel=-1)
			return 1

		except PortagePackageException as e:
			portage.writemsg(("!!! %s\n") % (e,), noiselevel=-1)
			return 1

		try:
			modified = mf.write(sign=False)
		except PermissionDenied as e:
			portage.writemsg("!!! %s: %s\n" % (_("Permission Denied"), e,),
				noiselevel=-1)
			return 1
		else:
			if modified:
				return self.MODIFIED
			return os.EX_OK



# xpak/test_decodeint.py
# Copyright 2006-2020 Gentoo Authors
# Portage Unit Testing Functionality

from portage.tests import TestCase
from portage.xpak import decodeint, encodeint

class testDecodeIntTestCase(TestCase):

	def testDecodeInt(self):

		for n in range(1000):
			self.assertEqual(decodeint(encodeint(n)), n)

		for n in (2 ** 32 - 1,):
			self.assertEqual(decodeint(encodeint(n)), n)



# Copyright 2011-2021 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

import argparse
import subprocess

import portage
from portage import shutil, os
from portage.const import (BASH_BINARY, BINREPOS_CONF_FILE, PORTAGE_PYM_PATH, USER_CONFIG_PATH)
from portage.cache.mappings import Mapping
from portage.process import find_binary
from portage.tests import TestCase
from portage.tests.resolver.ResolverPlayground import ResolverPlayground
from portage.tests.util.test_socks5 import AsyncHTTPServer
from portage.util import (ensure_dirs, find_updated_config_files,
	shlex_split)
from portage.util.futures import asyncio


class BinhostContentMap(Mapping):
	def __init__(self, remote_path, local_path):
		self._remote_path = remote_path
		self._local_path = local_path

	def __getitem__(self, request_path):
		safe_path = os.path.normpath(request_path)
		if not safe_path.startswith(self._remote_path + '/'):
			raise KeyError(request_path)
		local_path = os.path.join(self._local_path, safe_path[len(self._remote_path)+1:])
		try:
			with open(local_path, 'rb') as f:
				return f.read()
		except EnvironmentError:
			raise KeyError(request_path)


class SimpleEmergeTestCase(TestCase):

	def _have_python_xml(self):
		try:
			__import__("xml.etree.ElementTree")
			__import__("xml.parsers.expat").parsers.expat.ExpatError
		except (AttributeError, ImportError):
			return False
		return True

	def testSimple(self):

		debug = False

		install_something = """
S="${WORKDIR}"

pkg_pretend() {
	einfo "called pkg_pretend for $CATEGORY/$PF"
}

src_install() {
	einfo "installing something..."
	insinto /usr/lib/${P}
	echo "blah blah blah" > "${T}"/regular-file
	doins "${T}"/regular-file
	dosym regular-file /usr/lib/${P}/symlink || die

	# Test CONFIG_PROTECT
	insinto /etc
	newins "${T}"/regular-file ${PN}-${SLOT%/*}

	# Test code for bug #381629, using a copyright symbol encoded with latin-1.
	# We use $(printf "\\xa9") rather than $'\\xa9', since printf apparently
	# works in any case, while $'\\xa9' transforms to \\xef\\xbf\\xbd under
	# some conditions. TODO: Find out why it transforms to \\xef\\xbf\\xbd when
	# running tests for Python 3.2 (even though it's bash that is ultimately
	# responsible for performing the transformation).
	local latin_1_dir=/usr/lib/${P}/latin-1-$(printf "\\xa9")-directory
	insinto "${latin_1_dir}"
	echo "blah blah blah" > "${T}"/latin-1-$(printf "\\xa9")-regular-file || die
	doins "${T}"/latin-1-$(printf "\\xa9")-regular-file
	dosym latin-1-$(printf "\\xa9")-regular-file ${latin_1_dir}/latin-1-$(printf "\\xa9")-symlink || die

	call_has_and_best_version
}

pkg_config() {
	einfo "called pkg_config for $CATEGORY/$PF"
}

pkg_info() {
	einfo "called pkg_info for $CATEGORY/$PF"
}

pkg_preinst() {
	if ! ___eapi_best_version_and_has_version_support_-b_-d_-r; then
		# The BROOT variable is unset during pkg_* phases for EAPI 7,
		# therefore best/has_version -b is expected to fail if we attempt
		# to call it for EAPI 7 here.
		call_has_and_best_version
	fi
}

call_has_and_best_version() {
	local root_arg
	if ___eapi_best_version_and_has_version_support_-b_-d_-r; then
		root_arg="-b"
	else
		root_arg="--host-root"
	fi
	einfo "called ${EBUILD_PHASE_FUNC} for $CATEGORY/$PF"
	einfo "EPREFIX=${EPREFIX}"
	einfo "PORTAGE_OVERRIDE_EPREFIX=${PORTAGE_OVERRIDE_EPREFIX}"
	einfo "ROOT=${ROOT}"
	einfo "EROOT=${EROOT}"
	einfo "SYSROOT=${SYSROOT}"
	einfo "ESYSROOT=${ESYSROOT}"
	einfo "BROOT=${BROOT}"
	# Test that has_version and best_version work correctly with
	# prefix (involves internal ROOT -> EROOT calculation in order
	# to support ROOT override via the environment with EAPIs 3
	# and later which support prefix).
	if has_version $CATEGORY/$PN:$SLOT ; then
		einfo "has_version detects an installed instance of $CATEGORY/$PN:$SLOT"
		einfo "best_version reports that the installed instance is $(best_version $CATEGORY/$PN:$SLOT)"
	else
		einfo "has_version does not detect an installed instance of $CATEGORY/$PN:$SLOT"
	fi
	if [[ ${EPREFIX} != ${PORTAGE_OVERRIDE_EPREFIX} ]] ; then
		if has_version ${root_arg} $CATEGORY/$PN:$SLOT ; then
			einfo "has_version ${root_arg} detects an installed instance of $CATEGORY/$PN:$SLOT"
			einfo "best_version ${root_arg} reports that the installed instance is $(best_version ${root_arg} $CATEGORY/$PN:$SLOT)"
		else
			einfo "has_version ${root_arg} does not detect an installed instance of $CATEGORY/$PN:$SLOT"
		fi
	fi
}

"""

		ebuilds = {
			"dev-libs/A-1": {
				"EAPI" : "5",
				"IUSE" : "+flag",
				"KEYWORDS": "x86",
				"LICENSE": "GPL-2",
				"MISC_CONTENT": install_something,
				"RDEPEND": "flag? ( dev-libs/B[flag] )",
			},
			"dev-libs/B-1": {
				"EAPI" : "5",
				"IUSE" : "+flag",
				"KEYWORDS": "x86",
				"LICENSE": "GPL-2",
				"MISC_CONTENT": install_something,
			},
			"dev-libs/C-1": {
				"EAPI" : "7",
				"KEYWORDS": "~x86",
				"RDEPEND": "dev-libs/D[flag]",
				"MISC_CONTENT": install_something,
			},
			"dev-libs/D-1": {
				"EAPI" : "7",
				"KEYWORDS": "~x86",
				"IUSE" : "flag",
				"MISC_CONTENT": install_something,
			},
			"virtual/foo-0": {
				"EAPI" : "5",
				"KEYWORDS": "x86",
				"LICENSE": "GPL-2",
			},
		}

		installed = {
			"dev-libs/A-1": {
				"EAPI" : "5",
				"IUSE" : "+flag",
				"KEYWORDS": "x86",
				"LICENSE": "GPL-2",
				"RDEPEND": "flag? ( dev-libs/B[flag] )",
				"USE": "flag",
			},
			"dev-libs/B-1": {
				"EAPI" : "5",
				"IUSE" : "+flag",
				"KEYWORDS": "x86",
				"LICENSE": "GPL-2",
				"USE": "flag",
			},
			"dev-libs/depclean-me-1": {
				"EAPI" : "5",
				"IUSE" : "",
				"KEYWORDS": "x86",
				"LICENSE": "GPL-2",
				"USE": "",
			},
			"app-misc/depclean-me-1": {
				"EAPI" : "5",
				"IUSE" : "",
				"KEYWORDS": "x86",
				"LICENSE": "GPL-2",
				"RDEPEND": "dev-libs/depclean-me",
				"USE": "",
			},
		}

		metadata_xml_files = (
			(
				"dev-libs/A",
				{
					"flags" : "<flag name='flag'>Description of how USE='flag' affects this package</flag>",
				},
			),
			(
				"dev-libs/B",
				{
					"flags" : "<flag name='flag'>Description of how USE='flag' affects this package</flag>",
				},
			),
		)

		playground = ResolverPlayground(
			ebuilds=ebuilds, installed=installed, debug=debug)

		loop = asyncio._wrap_loop()
		loop.run_until_complete(asyncio.ensure_future(
			self._async_test_simple(playground, metadata_xml_files, loop=loop), loop=loop))

	async def _async_test_simple(self, playground, metadata_xml_files, loop):

		debug = playground.debug
		settings = playground.settings
		eprefix = settings["EPREFIX"]
		eroot = settings["EROOT"]
		trees = playground.trees
		portdb = trees[eroot]["porttree"].dbapi
		test_repo_location = settings.repositories["test_repo"].location
		var_cache_edb = os.path.join(eprefix, "var", "cache", "edb")
		cachedir = os.path.join(var_cache_edb, "dep")
		cachedir_pregen = os.path.join(test_repo_location, "metadata", "md5-cache")

		portage_python = portage._python_interpreter
		dispatch_conf_cmd = (portage_python, "-b", "-Wd",
			os.path.join(self.sbindir, "dispatch-conf"))
		ebuild_cmd = (portage_python, "-b", "-Wd",
			os.path.join(self.bindir, "ebuild"))
		egencache_cmd = (portage_python, "-b", "-Wd",
			os.path.join(self.bindir, "egencache"),
			"--repo", "test_repo",
			"--repositories-configuration", settings.repositories.config_string())
		emerge_cmd = (portage_python, "-b", "-Wd",
			os.path.join(self.bindir, "emerge"))
		emaint_cmd = (portage_python, "-b", "-Wd",
			os.path.join(self.sbindir, "emaint"))
		env_update_cmd = (portage_python, "-b", "-Wd",
			os.path.join(self.sbindir, "env-update"))
		etc_update_cmd = (BASH_BINARY,
			os.path.join(self.sbindir, "etc-update"))
		fixpackages_cmd = (portage_python, "-b", "-Wd",
			os.path.join(self.sbindir, "fixpackages"))
		portageq_cmd = (portage_python, "-b", "-Wd",
			os.path.join(self.bindir, "portageq"))
		quickpkg_cmd = (portage_python, "-b", "-Wd",
			os.path.join(self.bindir, "quickpkg"))
		regenworld_cmd = (portage_python, "-b", "-Wd",
			os.path.join(self.sbindir, "regenworld"))

		rm_binary = find_binary("rm")
		self.assertEqual(rm_binary is None, False,
			"rm command not found")
		rm_cmd = (rm_binary,)

		egencache_extra_args = []
		if self._have_python_xml():
			egencache_extra_args.append("--update-use-local-desc")

		test_ebuild = portdb.findname("dev-libs/A-1")
		self.assertFalse(test_ebuild is None)

		cross_prefix = os.path.join(eprefix, "cross_prefix")
		cross_root = os.path.join(eprefix, "cross_root")
		cross_eroot = os.path.join(cross_root, eprefix.lstrip(os.sep))

		binhost_dir = os.path.join(eprefix, "binhost")
		binhost_address = '127.0.0.1'
		binhost_remote_path = '/binhost'
		binhost_server = AsyncHTTPServer(binhost_address,
			BinhostContentMap(binhost_remote_path, binhost_dir), loop).__enter__()
		binhost_uri = 'http://{address}:{port}{path}'.format(
			address=binhost_address,
			port=binhost_server.server_port,
			path=binhost_remote_path)

		test_commands = ()

		if hasattr(argparse.ArgumentParser, "parse_intermixed_args"):
			test_commands += (
				emerge_cmd + ("--oneshot", "dev-libs/A", "-v", "dev-libs/A"),
			)

		test_commands += (
			emerge_cmd + ("--usepkgonly", "--root", cross_root, "--quickpkg-direct=y", "--quickpkg-direct-root", "/", "dev-libs/A"),
			emerge_cmd + ("--usepkgonly", "--quickpkg-direct=y", "--quickpkg-direct-root", cross_root, "dev-libs/A"),
			env_update_cmd,
			portageq_cmd + ("envvar", "-v", "CONFIG_PROTECT", "EROOT",
				"PORTAGE_CONFIGROOT", "PORTAGE_TMPDIR", "USERLAND"),
			etc_update_cmd,
			dispatch_conf_cmd,
			emerge_cmd + ("--version",),
			emerge_cmd + ("--info",),
			emerge_cmd + ("--info", "--verbose"),
			emerge_cmd + ("--list-sets",),
			emerge_cmd + ("--check-news",),
			rm_cmd + ("-rf", cachedir),
			rm_cmd + ("-rf", cachedir_pregen),
			emerge_cmd + ("--regen",),
			rm_cmd + ("-rf", cachedir),
			({"FEATURES" : "metadata-transfer"},) + \
				emerge_cmd + ("--regen",),
			rm_cmd + ("-rf", cachedir),
			({"FEATURES" : "metadata-transfer"},) + \
				emerge_cmd + ("--regen",),
			rm_cmd + ("-rf", cachedir),
			egencache_cmd + ("--update",) + tuple(egencache_extra_args),
			({"FEATURES" : "metadata-transfer"},) + \
				emerge_cmd + ("--metadata",),
			rm_cmd + ("-rf", cachedir),
			({"FEATURES" : "metadata-transfer"},) + \
				emerge_cmd + ("--metadata",),
			emerge_cmd + ("--metadata",),
			rm_cmd + ("-rf", cachedir),
			emerge_cmd + ("--oneshot", "virtual/foo"),
			lambda: self.assertFalse(os.path.exists(
				os.path.join(pkgdir, "virtual", "foo", "foo-0-1.xpak"))),
			({"FEATURES" : "unmerge-backup"},) + \
				emerge_cmd + ("--unmerge", "virtual/foo"),
			lambda: self.assertTrue(os.path.exists(
				os.path.join(pkgdir, "virtual", "foo", "foo-0-1.xpak"))),
			emerge_cmd + ("--pretend", "dev-libs/A"),
			ebuild_cmd + (test_ebuild, "manifest", "clean", "package", "merge"),
			emerge_cmd + ("--pretend", "--tree", "--complete-graph", "dev-libs/A"),
			emerge_cmd + ("-p", "dev-libs/B"),
			emerge_cmd + ("-p", "--newrepo", "dev-libs/B"),
			emerge_cmd + ("-B", "dev-libs/B",),
			emerge_cmd + ("--oneshot", "--usepkg", "dev-libs/B",),

			# trigger clean prior to pkg_pretend as in bug #390711
			ebuild_cmd + (test_ebuild, "unpack"),
			emerge_cmd + ("--oneshot", "dev-libs/A",),

			emerge_cmd + ("--noreplace", "dev-libs/A",),
			emerge_cmd + ("--config", "dev-libs/A",),
			emerge_cmd + ("--info", "dev-libs/A", "dev-libs/B"),
			emerge_cmd + ("--pretend", "--depclean", "--verbose", "dev-libs/B"),
			emerge_cmd + ("--pretend", "--depclean",),
			emerge_cmd + ("--depclean",),
			quickpkg_cmd + ("--include-config", "y", "dev-libs/A",),
			# Test bug #523684, where a file renamed or removed by the
			# admin forces replacement files to be merged with config
			# protection.
			lambda: self.assertEqual(0,
				len(list(find_updated_config_files(eroot,
				shlex_split(settings["CONFIG_PROTECT"]))))),
			lambda: os.unlink(os.path.join(eprefix, "etc", "A-0")),
			emerge_cmd + ("--usepkgonly", "dev-libs/A"),
			lambda: self.assertEqual(1,
				len(list(find_updated_config_files(eroot,
				shlex_split(settings["CONFIG_PROTECT"]))))),
			emaint_cmd + ("--check", "all"),
			emaint_cmd + ("--fix", "all"),
			fixpackages_cmd,
			regenworld_cmd,
			portageq_cmd + ("match", eroot, "dev-libs/A"),
			portageq_cmd + ("best_visible", eroot, "dev-libs/A"),
			portageq_cmd + ("best_visible", eroot, "binary", "dev-libs/A"),
			portageq_cmd + ("contents", eroot, "dev-libs/A-1"),
			portageq_cmd + ("metadata", eroot, "ebuild", "dev-libs/A-1", "EAPI", "IUSE", "RDEPEND"),
			portageq_cmd + ("metadata", eroot, "binary", "dev-libs/A-1", "EAPI", "USE", "RDEPEND"),
			portageq_cmd + ("metadata", eroot, "installed", "dev-libs/A-1", "EAPI", "USE", "RDEPEND"),
			portageq_cmd + ("owners", eroot, eroot + "usr"),
			emerge_cmd + ("-p", eroot + "usr"),
			emerge_cmd + ("-p", "--unmerge", "-q", eroot + "usr"),
			emerge_cmd + ("--unmerge", "--quiet", "dev-libs/A"),
			emerge_cmd + ("-C", "--quiet", "dev-libs/B"),

			# If EMERGE_DEFAULT_OPTS contains --autounmask=n, then --autounmask
			# must be specified with --autounmask-continue.
			({"EMERGE_DEFAULT_OPTS" : "--autounmask=n"},) + \
				emerge_cmd + ("--autounmask", "--autounmask-continue", "dev-libs/C",),
			# Verify that the above --autounmask-continue command caused
			# USE=flag to be applied correctly to dev-libs/D.
			portageq_cmd + ("match", eroot, "dev-libs/D[flag]"),

			# Test cross-prefix usage, including chpathtool for binpkgs.
			# EAPI 7
			({"EPREFIX" : cross_prefix},) + \
				emerge_cmd + ("dev-libs/C",),
			({"EPREFIX" : cross_prefix},) + \
				portageq_cmd + ("has_version", cross_prefix, "dev-libs/C"),
			({"EPREFIX" : cross_prefix},) + \
				portageq_cmd + ("has_version", cross_prefix, "dev-libs/D"),
			({"ROOT": cross_root},) + emerge_cmd + ("dev-libs/D",),
			portageq_cmd + ("has_version", cross_eroot, "dev-libs/D"),
			# EAPI 5
			({"EPREFIX" : cross_prefix},) + \
				emerge_cmd + ("--usepkgonly", "dev-libs/A"),
			({"EPREFIX" : cross_prefix},) + \
				portageq_cmd + ("has_version", cross_prefix, "dev-libs/A"),
			({"EPREFIX" : cross_prefix},) + \
				portageq_cmd + ("has_version", cross_prefix, "dev-libs/B"),
			({"EPREFIX" : cross_prefix},) + \
				emerge_cmd + ("-C", "--quiet", "dev-libs/B"),
			({"EPREFIX" : cross_prefix},) + \
				emerge_cmd + ("-C", "--quiet", "dev-libs/A"),
			({"EPREFIX" : cross_prefix},) + \
				emerge_cmd + ("dev-libs/A",),
			({"EPREFIX" : cross_prefix},) + \
				portageq_cmd + ("has_version", cross_prefix, "dev-libs/A"),
			({"EPREFIX" : cross_prefix},) + \
				portageq_cmd + ("has_version", cross_prefix, "dev-libs/B"),

			# Test ROOT support
			({"ROOT": cross_root},) + emerge_cmd + ("dev-libs/B",),
			portageq_cmd + ("has_version", cross_eroot, "dev-libs/B"),
		)

		# Test binhost support if FETCHCOMMAND is available.
		binrepos_conf_file = os.path.join(os.sep, eprefix, BINREPOS_CONF_FILE)
		with open(binrepos_conf_file, 'wt') as f:
			f.write('[test-binhost]\n')
			f.write('sync-uri = {}\n'.format(binhost_uri))
		fetchcommand = portage.util.shlex_split(playground.settings['FETCHCOMMAND'])
		fetch_bin = portage.process.find_binary(fetchcommand[0])
		if fetch_bin is not None:
			test_commands = test_commands + (
				lambda: os.rename(pkgdir, binhost_dir),
				emerge_cmd + ("-e", "--getbinpkgonly", "dev-libs/A"),
				lambda: shutil.rmtree(pkgdir),
				lambda: os.rename(binhost_dir, pkgdir),
				# Remove binrepos.conf and test PORTAGE_BINHOST.
				lambda: os.unlink(binrepos_conf_file),
				lambda: os.rename(pkgdir, binhost_dir),
				({"PORTAGE_BINHOST": binhost_uri},) + \
					emerge_cmd + ("-fe", "--getbinpkgonly", "dev-libs/A"),
				lambda: shutil.rmtree(pkgdir),
				lambda: os.rename(binhost_dir, pkgdir),
			)

		distdir = playground.distdir
		pkgdir = playground.pkgdir
		fake_bin = os.path.join(eprefix, "bin")
		portage_tmpdir = os.path.join(eprefix, "var", "tmp", "portage")
		profile_path = settings.profile_path
		user_config_dir = os.path.join(os.sep, eprefix, USER_CONFIG_PATH)

		path =  os.environ.get("PATH")
		if path is not None and not path.strip():
			path = None
		if path is None:
			path = ""
		else:
			path = ":" + path
		path = fake_bin + path

		pythonpath =  os.environ.get("PYTHONPATH")
		if pythonpath is not None and not pythonpath.strip():
			pythonpath = None
		if pythonpath is not None and \
			pythonpath.split(":")[0] == PORTAGE_PYM_PATH:
			pass
		else:
			if pythonpath is None:
				pythonpath = ""
			else:
				pythonpath = ":" + pythonpath
			pythonpath = PORTAGE_PYM_PATH + pythonpath

		env = {
			"PORTAGE_OVERRIDE_EPREFIX" : eprefix,
			"CLEAN_DELAY" : "0",
			"DISTDIR" : distdir,
			"EMERGE_WARNING_DELAY" : "0",
			"INFODIR" : "",
			"INFOPATH" : "",
			"PATH" : path,
			"PKGDIR" : pkgdir,
			"PORTAGE_INST_GID" : str(portage.data.portage_gid),
			"PORTAGE_INST_UID" : str(portage.data.portage_uid),
			"PORTAGE_PYTHON" : portage_python,
			"PORTAGE_REPOSITORIES" : settings.repositories.config_string(),
			"PORTAGE_TMPDIR" : portage_tmpdir,
			"PORTAGE_LOGDIR" : portage_tmpdir,
			"PYTHONDONTWRITEBYTECODE" : os.environ.get("PYTHONDONTWRITEBYTECODE", ""),
			"PYTHONPATH" : pythonpath,
			"__PORTAGE_TEST_PATH_OVERRIDE" : fake_bin,
		}

		if "__PORTAGE_TEST_HARDLINK_LOCKS" in os.environ:
			env["__PORTAGE_TEST_HARDLINK_LOCKS"] = \
				os.environ["__PORTAGE_TEST_HARDLINK_LOCKS"]

		updates_dir = os.path.join(test_repo_location, "profiles", "updates")
		dirs = [cachedir, cachedir_pregen, cross_eroot, cross_prefix,
			distdir, fake_bin, portage_tmpdir, updates_dir,
			user_config_dir, var_cache_edb]
		etc_symlinks = ("dispatch-conf.conf", "etc-update.conf")
		# Override things that may be unavailable, or may have portability
		# issues when running tests in exotic environments.
		#   prepstrip - bug #447810 (bash read builtin EINTR problem)
		true_symlinks = ["find", "prepstrip", "sed", "scanelf"]
		true_binary = find_binary("true")
		self.assertEqual(true_binary is None, False,
			"true command not found")
		try:
			for d in dirs:
				ensure_dirs(d)
			for x in true_symlinks:
				os.symlink(true_binary, os.path.join(fake_bin, x))
			for x in etc_symlinks:
				os.symlink(os.path.join(self.cnf_etc_path, x),
					os.path.join(eprefix, "etc", x))
			with open(os.path.join(var_cache_edb, "counter"), 'wb') as f:
				f.write(b"100")
			# non-empty system set keeps --depclean quiet
			with open(os.path.join(profile_path, "packages"), 'w') as f:
				f.write("*dev-libs/token-system-pkg")
			for cp, xml_data in metadata_xml_files:
				with open(os.path.join(test_repo_location, cp, "metadata.xml"), 'w') as f:
					f.write(playground.metadata_xml_template % xml_data)
			with open(os.path.join(updates_dir, "1Q-2010"), 'w') as f:
				f.write("""
slotmove =app-doc/pms-3 2 3
move dev-util/git dev-vcs/git
""")

			if debug:
				# The subprocess inherits both stdout and stderr, for
				# debugging purposes.
				stdout = None
			else:
				# The subprocess inherits stderr so that any warnings
				# triggered by python -Wd will be visible.
				stdout = subprocess.PIPE

			for args in test_commands:

				if hasattr(args, '__call__'):
					args()
					continue

				if isinstance(args[0], dict):
					local_env = env.copy()
					local_env.update(args[0])
					args = args[1:]
				else:
					local_env = env

				proc = await asyncio.create_subprocess_exec(*args,
					env=local_env, stderr=None, stdout=stdout)

				if debug:
					await proc.wait()
				else:
					output, _err = await proc.communicate()
					await proc.wait()
					if proc.returncode != os.EX_OK:
						portage.writemsg(output)

				self.assertEqual(os.EX_OK, proc.returncode,
					"emerge failed with args %s" % (args,))
		finally:
			binhost_server.__exit__(None, None, None)
			playground.cleanup()



# Copyright 1999-2020 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

from collections import deque

from portage import os
from _emerge.CompositeTask import CompositeTask
from _emerge.AsynchronousTask import AsynchronousTask

class TaskSequence(CompositeTask):
	"""
	A collection of tasks that executes sequentially. Each task
	must have a addExitListener() method that can be used as
	a means to trigger movement from one task to the next.
	"""

	__slots__ = ("_task_queue",)

	def __init__(self, **kwargs):
		AsynchronousTask.__init__(self, **kwargs)
		self._task_queue = deque()

	def add(self, task):
		self._task_queue.append(task)

	def _start(self):
		self._start_next_task()

	def _cancel(self):
		self._task_queue.clear()
		CompositeTask._cancel(self)

	def _start_next_task(self):
		try:
			task = self._task_queue.popleft()
		except IndexError:
			self._current_task = None
			self.returncode = os.EX_OK
			self.wait()
			return

		self._start_task(task, self._task_exit_handler)

	def _task_exit_handler(self, task):
		if self._default_exit(task) != os.EX_OK:
			self.wait()
		elif self._task_queue:
			self._start_next_task()
		else:
			self._final_exit(task)
			self.wait()

	def __bool__(self):
		return bool(self._task_queue)

	def __len__(self):
		return len(self._task_queue)



# Copyright 1999-2020 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

import logging
import textwrap

import portage
from portage import os
from portage.emaint.modules.logs.logs import CleanLogs
from portage.news import count_unread_news, display_news_notifications
from portage.output import colorize
from portage.util._dyn_libs.display_preserved_libs import \
	display_preserved_libs
from portage.util._info_files import chk_updated_info_files

from .chk_updated_cfg_files import chk_updated_cfg_files
from .emergelog import emergelog
from ._flush_elog_mod_echo import _flush_elog_mod_echo

def clean_logs(settings):

	if "clean-logs" not in settings.features:
		return

	logdir = settings.get("PORTAGE_LOGDIR")
	if logdir is None or not os.path.isdir(logdir):
		return

	cleanlogs = CleanLogs()
	returncode, msgs = cleanlogs.clean(settings=settings)
	if not returncode:
		out = portage.output.EOutput()
		for msg in msgs:
			out.eerror(msg)

def display_news_notification(root_config, myopts):
	if "news" not in root_config.settings.features:
		return False
	portdb = root_config.trees["porttree"].dbapi
	vardb = root_config.trees["vartree"].dbapi
	news_counts = count_unread_news(portdb, vardb)
	if all(v == 0 for v in news_counts.values()):
		return False
	display_news_notifications(news_counts)
	return True

def show_depclean_suggestion():
	out = portage.output.EOutput()
	msg = "After world updates, it is important to remove " + \
		"obsolete packages with emerge --depclean. Refer " + \
		"to `man emerge` for more information."
	for line in textwrap.wrap(msg, 72):
		out.ewarn(line)

def post_emerge(myaction, myopts, myfiles,
	target_root, trees, mtimedb, retval):
	"""
	Misc. things to run at the end of a merge session.

	Update Info Files
	Update Config Files
	Update News Items
	Commit mtimeDB
	Display preserved libs warnings

	@param myaction: The action returned from parse_opts()
	@type myaction: String
	@param myopts: emerge options
	@type myopts: dict
	@param myfiles: emerge arguments
	@type myfiles: list
	@param target_root: The target EROOT for myaction
	@type target_root: String
	@param trees: A dictionary mapping each ROOT to it's package databases
	@type trees: dict
	@param mtimedb: The mtimeDB to store data needed across merge invocations
	@type mtimedb: MtimeDB class instance
	@param retval: Emerge's return value
	@type retval: Int
	"""

	root_config = trees[target_root]["root_config"]
	vardbapi = trees[target_root]['vartree'].dbapi
	settings = vardbapi.settings
	info_mtimes = mtimedb["info"]

	# Load the most current variables from ${ROOT}/etc/profile.env
	settings.unlock()
	settings.reload()
	settings.regenerate()
	settings.lock()

	config_protect = portage.util.shlex_split(
		settings.get("CONFIG_PROTECT", ""))
	infodirs = settings.get("INFOPATH","").split(":") + \
		settings.get("INFODIR","").split(":")

	os.chdir("/")

	if retval == os.EX_OK:
		exit_msg = " *** exiting successfully."
	else:
		exit_msg = " *** exiting unsuccessfully with status '%s'." % retval
	emergelog("notitles" not in settings.features, exit_msg)

	_flush_elog_mod_echo()

	if not vardbapi._pkgs_changed:
		# GLEP 42 says to display news *after* an emerge --pretend
		if "--pretend" in myopts:
			display_news_notification(root_config, myopts)
		# If vdb state has not changed then there's nothing else to do.
		return

	vdb_path = os.path.join(root_config.settings['EROOT'], portage.VDB_PATH)
	portage.util.ensure_dirs(vdb_path)
	vdb_lock = None
	if os.access(vdb_path, os.W_OK) and not "--pretend" in myopts:
		vardbapi.lock()
		vdb_lock = True

	if vdb_lock:
		try:
			if "noinfo" not in settings.features:
				chk_updated_info_files(target_root,
					infodirs, info_mtimes)
			mtimedb.commit()
		finally:
			if vdb_lock:
				vardbapi.unlock()

	# Explicitly load and prune the PreservedLibsRegistry in order
	# to ensure that we do not display stale data.
	vardbapi._plib_registry.load()

	if vardbapi._plib_registry.hasEntries():
		if "--quiet" in myopts:
			print()
			print(colorize("WARN", "!!!") + " existing preserved libs found")
		else:
			print()
			print(colorize("WARN", "!!!") + " existing preserved libs:")
			display_preserved_libs(vardbapi)
			print("Use " + colorize("GOOD", "emerge @preserved-rebuild") +
				" to rebuild packages using these libraries")

	chk_updated_cfg_files(settings['EROOT'], config_protect)

	display_news_notification(root_config, myopts)

	postemerge = os.path.join(settings["PORTAGE_CONFIGROOT"],
		portage.USER_CONFIG_PATH, "bin", "post_emerge")
	if os.access(postemerge, os.X_OK):
		hook_retval = portage.process.spawn(
						[postemerge], env=settings.environ())
		if hook_retval != os.EX_OK:
			portage.util.writemsg_level(
				" %s spawn failed of %s\n" %
				(colorize("BAD", "*"), postemerge,),
				level=logging.ERROR, noiselevel=-1)

	clean_logs(settings)

	if "--quiet" not in myopts and \
		myaction is None and "@world" in myfiles:
		show_depclean_suggestion()



# Copyright 2018 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

import time
from portage.tests import TestCase
from portage.util._async.ForkProcess import ForkProcess
from portage.util._eventloop.global_event_loop import global_event_loop
from portage.util.futures import asyncio
from portage.util.futures.iter_completed import (
	iter_completed,
	async_iter_completed,
)


class SleepProcess(ForkProcess):
	__slots__ = ('future', 'seconds')
	def _start(self):
		self.addExitListener(self._future_done)
		ForkProcess._start(self)

	def _future_done(self, task):
		if not self.future.cancelled():
			self.future.set_result(self.seconds)

	def _run(self):
		time.sleep(self.seconds)


class IterCompletedTestCase(TestCase):

	def testIterCompleted(self):

		# Mark this as todo, since we don't want to fail if heavy system
		# load causes the tasks to finish in an unexpected order.
		self.todo = True

		loop = global_event_loop()
		tasks = [
			SleepProcess(seconds=0.200),
			SleepProcess(seconds=0.100),
			SleepProcess(seconds=0.001),
		]

		expected_order = sorted(task.seconds for task in tasks)

		def future_generator():
			for task in tasks:
				task.future = loop.create_future()
				task.scheduler = loop
				task.start()
				yield task.future

		for seconds, future in zip(expected_order, iter_completed(future_generator(),
			max_jobs=True, max_load=None, loop=loop)):
			self.assertEqual(seconds, future.result())

	def testAsyncCancel(self):

		loop = global_event_loop()
		input_futures = set()
		future_count = 3

		def future_generator():
			for i in range(future_count):
				future = loop.create_future()
				loop.call_soon(lambda future: None if future.done()
					else future.set_result(None), future)
				input_futures.add(future)
				yield future

		for future_done_set in async_iter_completed(future_generator(),
			max_jobs=True, max_load=True, loop=loop):
			future_done_set.cancel()
			break

		# With max_jobs=True, async_iter_completed should have executed
		# the generator until it raised StopIteration.
		self.assertEqual(future_count, len(input_futures))

		loop.run_until_complete(asyncio.wait(input_futures, loop=loop))

		# The futures may have results or they may have been cancelled
		# by TaskScheduler, and behavior varies depending on the python
		# interpreter.
		for future in input_futures:
			future.cancelled() or future.result()



# Copyright 2014 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

from portage.tests import TestCase
from portage.tests.resolver.ResolverPlayground import (ResolverPlayground,
	ResolverPlaygroundTestCase)

class SlotOperatorRequiredUseTestCase(TestCase):

	def testSlotOperatorRequiredUse(self):

		ebuilds = {

			"app-misc/A-1" : {
				"EAPI": "5",
				"SLOT": "0/1"
			},

			"app-misc/A-2" : {
				"EAPI": "5",
				"SLOT": "0/2"
			},

			"app-misc/B-0" : {
				"EAPI": "5",
				"RDEPEND": "app-misc/A:=",
				"IUSE": "x y",
				"REQUIRED_USE": "|| ( x y )"
			},

		}

		installed = {

			"app-misc/A-1" : {
				"EAPI": "5",
				"SLOT": "0/1"
			},

			"app-misc/B-0" : {
				"EAPI": "5",
				"RDEPEND": "app-misc/A:0/1=",
				"IUSE": "x y",
				"USE": "x"
			},

		}

		world = ["app-misc/B"]

		test_cases = (

			# bug 523048
			# Ensure that unsatisfied REQUIRED_USE is reported when
			# it blocks necessary slot-operator rebuilds.
			ResolverPlaygroundTestCase(
				["app-misc/A"],
				success = False,
				required_use_unsatisfied = ['app-misc/B:0']
			),

		)

		playground = ResolverPlayground(ebuilds=ebuilds,
			installed=installed, world=world, debug=False)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True,
					test_case.fail_msg)
		finally:
			playground.cleanup()



# Copyright 1998-2013 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

import errno
import io
import tempfile
import portage
from portage import os
from portage import _encodings
from portage import _unicode_encode
from portage.const import BASH_BINARY
from portage.tests import TestCase
from portage.util._eventloop.global_event_loop import global_event_loop
from _emerge.SpawnProcess import SpawnProcess

class SpawnTestCase(TestCase):

	def testLogfile(self):
		logfile = None
		try:
			fd, logfile = tempfile.mkstemp()
			os.close(fd)
			null_fd = os.open('/dev/null', os.O_RDWR)
			test_string = 2 * "blah blah blah\n"
			proc = SpawnProcess(
				args=[BASH_BINARY, "-c",
				"echo -n '%s'" % test_string],
				env={},
				fd_pipes={
					0: portage._get_stdin().fileno(),
					1: null_fd,
					2: null_fd
				},
				scheduler=global_event_loop(),
				logfile=logfile)
			proc.start()
			os.close(null_fd)
			self.assertEqual(proc.wait(), os.EX_OK)
			f = io.open(_unicode_encode(logfile,
				encoding=_encodings['fs'], errors='strict'),
				mode='r', encoding=_encodings['content'], errors='strict')
			log_content = f.read()
			f.close()
			# When logging passes through a pty, this comparison will fail
			# unless the oflag terminal attributes have the termios.OPOST
			# bit disabled. Otherwise, tranformations such as \n -> \r\n
			# may occur.
			self.assertEqual(test_string, log_content)
		finally:
			if logfile:
				try:
					os.unlink(logfile)
				except EnvironmentError as e:
					if e.errno != errno.ENOENT:
						raise
					del e



# Copyright 1999-2014 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

from portage import os
from portage.dep import _repo_separator
from portage.output import colorize

from _emerge.AsynchronousTask import AsynchronousTask
from _emerge.Binpkg import Binpkg
from _emerge.CompositeTask import CompositeTask
from _emerge.EbuildBuild import EbuildBuild
from _emerge.PackageUninstall import PackageUninstall

class MergeListItem(CompositeTask):

	"""
	TODO: For parallel scheduling, everything here needs asynchronous
	execution support (start, poll, and wait methods).
	"""

	__slots__ = ("args_set",
		"binpkg_opts", "build_opts", "config_pool", "emerge_opts",
		"find_blockers", "logger", "mtimedb", "pkg",
		"pkg_count", "pkg_to_replace", "prefetcher",
		"settings", "statusMessage", "world_atom") + \
		("_install_task",)

	def _start(self):

		pkg = self.pkg
		build_opts = self.build_opts

		if pkg.installed:
			# uninstall,  executed by self.merge()
			self.returncode = os.EX_OK
			self._async_wait()
			return

		args_set = self.args_set
		find_blockers = self.find_blockers
		logger = self.logger
		mtimedb = self.mtimedb
		pkg_count = self.pkg_count
		scheduler = self.scheduler
		settings = self.settings
		world_atom = self.world_atom
		ldpath_mtimes = mtimedb["ldpath"]

		action_desc = "Emerging"
		preposition = "for"
		pkg_color = "PKG_MERGE"
		if pkg.type_name == "binary":
			pkg_color = "PKG_BINARY_MERGE"
			action_desc += " binary"

		if build_opts.fetchonly:
			action_desc = "Fetching"

		msg = "%s (%s of %s) %s" % \
			(action_desc,
			colorize("MERGE_LIST_PROGRESS", str(pkg_count.curval)),
			colorize("MERGE_LIST_PROGRESS", str(pkg_count.maxval)),
			colorize(pkg_color, pkg.cpv + _repo_separator + pkg.repo))

		if pkg.root_config.settings["ROOT"] != "/":
			msg += " %s %s" % (preposition, pkg.root)

		if not build_opts.pretend:
			self.statusMessage(msg)
			logger.log(" >>> emerge (%s of %s) %s to %s" % \
				(pkg_count.curval, pkg_count.maxval, pkg.cpv, pkg.root))

		if pkg.type_name == "ebuild":

			build = EbuildBuild(args_set=args_set,
				background=self.background,
				config_pool=self.config_pool,
				find_blockers=find_blockers,
				ldpath_mtimes=ldpath_mtimes, logger=logger,
				opts=build_opts, pkg=pkg, pkg_count=pkg_count,
				prefetcher=self.prefetcher, scheduler=scheduler,
				settings=settings, world_atom=world_atom)

			self._install_task = build
			self._start_task(build, self._default_final_exit)
			return

		if pkg.type_name == "binary":

			binpkg = Binpkg(background=self.background,
				find_blockers=find_blockers,
				ldpath_mtimes=ldpath_mtimes, logger=logger,
				opts=self.binpkg_opts, pkg=pkg, pkg_count=pkg_count,
				prefetcher=self.prefetcher, settings=settings,
				scheduler=scheduler, world_atom=world_atom)

			self._install_task = binpkg
			self._start_task(binpkg, self._default_final_exit)
			return

	def create_install_task(self):

		pkg = self.pkg
		build_opts = self.build_opts
		mtimedb = self.mtimedb
		scheduler = self.scheduler
		settings = self.settings
		world_atom = self.world_atom
		ldpath_mtimes = mtimedb["ldpath"]

		if pkg.installed:
			if not (build_opts.buildpkgonly or \
				build_opts.fetchonly or build_opts.pretend):

				task = PackageUninstall(background=self.background,
					ldpath_mtimes=ldpath_mtimes, opts=self.emerge_opts,
					pkg=pkg, scheduler=scheduler, settings=settings,
					world_atom=world_atom)

			else:
				task = AsynchronousTask()

		elif build_opts.fetchonly or \
			build_opts.buildpkgonly:
			task = AsynchronousTask()
		else:
			task = self._install_task.create_install_task()

		return task



# Copyright 2010 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

__all__ = (
	'VirtualsManager',
)

from copy import deepcopy

from portage import os
from portage.dep import Atom
from portage.exception import InvalidAtom
from portage.localization import _
from portage.util import grabdict, stack_dictlist, writemsg
from portage.versions import cpv_getkey

class VirtualsManager:

	def __init__(self, *args, **kwargs):
		if kwargs.get("_copy"):
			return

		assert len(args) == 1, "VirtualsManager.__init__ takes one positional argument"
		assert not kwargs, "unknown keyword argument(s) '%s' passed to VirtualsManager.__init__" % \
			", ".join(kwargs)

		profiles = args[0]
		self._virtuals = None
		self._dirVirtuals = None
		self._virts_p = None

		# Virtuals obtained from the vartree
		self._treeVirtuals = None
		# Virtuals added by the depgraph via self.add_depgraph_virtuals().
		self._depgraphVirtuals = {}

		#Initialise _dirVirtuals.
		self._read_dirVirtuals(profiles)

		#We could initialise _treeVirtuals here, but some consumers want to
		#pass their own vartree.

	def _read_dirVirtuals(self, profiles):
		"""
		Read the 'virtuals' file in all profiles.
		"""
		virtuals_list = []
		for x in profiles:
			virtuals_file = os.path.join(x, "virtuals")
			virtuals_dict = grabdict(virtuals_file)
			atoms_dict = {}
			for k, v in virtuals_dict.items():
				try:
					virt_atom = Atom(k)
				except InvalidAtom:
					virt_atom = None
				else:
					if virt_atom.blocker or \
						str(virt_atom) != str(virt_atom.cp):
						virt_atom = None
				if virt_atom is None:
					writemsg(_("--- Invalid virtuals atom in %s: %s\n") % \
						(virtuals_file, k), noiselevel=-1)
					continue
				providers = []
				for atom in v:
					atom_orig = atom
					if atom[:1] == '-':
						# allow incrementals
						atom = atom[1:]
					try:
						atom = Atom(atom)
					except InvalidAtom:
						atom = None
					else:
						if atom.blocker:
							atom = None
					if atom is None:
						writemsg(_("--- Invalid atom in %s: %s\n") % \
							(virtuals_file, atom_orig), noiselevel=-1)
					else:
						if atom_orig == str(atom):
							# normal atom, so return as Atom instance
							providers.append(atom)
						else:
							# atom has special prefix, so return as string
							providers.append(atom_orig)
				if providers:
					atoms_dict[virt_atom] = providers
			if atoms_dict:
				virtuals_list.append(atoms_dict)

		self._dirVirtuals = stack_dictlist(virtuals_list, incremental=True)

		for virt in self._dirVirtuals:
			# Preference for virtuals decreases from left to right.
			self._dirVirtuals[virt].reverse()

	def __deepcopy__(self, memo=None):
		if memo is None:
			memo = {}
		result = VirtualsManager(_copy=True)
		memo[id(self)] = result

		# immutable attributes (internal policy ensures lack of mutation)
		# _treeVirtuals is initilised by _populate_treeVirtuals().
		# Before that it's 'None'.
		result._treeVirtuals = self._treeVirtuals
		memo[id(self._treeVirtuals)] = self._treeVirtuals
		# _dirVirtuals is initilised by __init__.
		result._dirVirtuals = self._dirVirtuals
		memo[id(self._dirVirtuals)] = self._dirVirtuals

		# mutable attributes (change when add_depgraph_virtuals() is called)
		result._virtuals = deepcopy(self._virtuals, memo)
		result._depgraphVirtuals = deepcopy(self._depgraphVirtuals, memo)
		result._virts_p = deepcopy(self._virts_p, memo)

		return result

	def _compile_virtuals(self):
		"""Stack installed and profile virtuals.  Preference for virtuals
		decreases from left to right.
		Order of preference:
		1. installed and in profile
		2. installed only
		3. profile only
		"""

		assert self._treeVirtuals is not None, "_populate_treeVirtuals() must be called before " + \
			"any query about virtuals"

		# Virtuals by profile+tree preferences.
		ptVirtuals   = {}

		for virt, installed_list in self._treeVirtuals.items():
			profile_list = self._dirVirtuals.get(virt, None)
			if not profile_list:
				continue
			for cp in installed_list:
				if cp in profile_list:
					ptVirtuals.setdefault(virt, [])
					ptVirtuals[virt].append(cp)

		virtuals = stack_dictlist([ptVirtuals, self._treeVirtuals,
			self._dirVirtuals, self._depgraphVirtuals])
		self._virtuals = virtuals
		self._virts_p = None

	def getvirtuals(self):
		"""
		Computes self._virtuals if necessary and returns it.
		self._virtuals is only computed on the first call.
		"""
		if self._virtuals is None:
			self._compile_virtuals()

		return self._virtuals

	def _populate_treeVirtuals(self, vartree):
		"""
		Initialize _treeVirtuals from the given vartree.
		It must not have been initialized already, otherwise
		our assumptions about immutability don't hold.
		"""
		assert self._treeVirtuals is None, "treeVirtuals must not be reinitialized"

		self._treeVirtuals = {}

		for provide, cpv_list in vartree.get_all_provides().items():
			try:
				provide = Atom(provide)
			except InvalidAtom:
				continue
			self._treeVirtuals[provide.cp] = \
				[Atom(cpv_getkey(cpv)) for cpv in cpv_list]

	def populate_treeVirtuals_if_needed(self, vartree):
		"""
		Initialize _treeVirtuals if it hasn't been done already.
		This is a hack for consumers that already have an populated vartree.
		"""
		if self._treeVirtuals is not None:
			return

		self._populate_treeVirtuals(vartree)

	def add_depgraph_virtuals(self, mycpv, virts):
		"""This updates the preferences for old-style virtuals,
		affecting the behavior of dep_expand() and dep_check()
		calls. It can change dbapi.match() behavior since that
		calls dep_expand(). However, dbapi instances have
		internal match caches that are not invalidated when
		preferences are updated here. This can potentially
		lead to some inconsistency (relevant to bug #1343)."""

		#Ensure that self._virtuals is populated.
		if self._virtuals is None:
			self.getvirtuals()

		modified = False
		cp = Atom(cpv_getkey(mycpv))
		for virt in virts:
			try:
				virt = Atom(virt).cp
			except InvalidAtom:
				continue
			providers = self._virtuals.get(virt)
			if providers and cp in providers:
				continue
			providers = self._depgraphVirtuals.get(virt)
			if providers is None:
				providers = []
				self._depgraphVirtuals[virt] = providers
			if cp not in providers:
				providers.append(cp)
				modified = True

		if modified:
			self._compile_virtuals()

	def get_virts_p(self):
		if self._virts_p is not None:
			return self._virts_p

		virts = self.getvirtuals()
		virts_p = {}
		for x in virts:
			vkeysplit = x.split("/")
			if vkeysplit[1] not in virts_p:
				virts_p[vkeysplit[1]] = virts[x]
		self._virts_p = virts_p
		return virts_p



# Copyright 2010-2021 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

__all__ = (
	'MaskManager',
)

import warnings

from portage import os
from portage.dep import ExtendedAtomDict, match_from_list
from portage.localization import _
from portage.repository.config import allow_profile_repo_deps
from portage.util import append_repo, grabfile_package, stack_lists, writemsg
from portage.versions import _pkg_str

class MaskManager:

	def __init__(self, repositories, profiles, abs_user_config,
		user_config=True, strict_umatched_removal=False):
		self._punmaskdict = ExtendedAtomDict(list)
		self._pmaskdict = ExtendedAtomDict(list)
		# Preserves atoms that are eliminated by negative
		# incrementals in user_pkgmasklines.
		self._pmaskdict_raw = ExtendedAtomDict(list)

		#Read profile/package.mask from every repo.
		#Repositories inherit masks from their parent profiles and
		#are able to remove mask from them with -atoms.
		#Such a removal affects only the current repo, but not the parent.
		#Add ::repo specs to every atom to make sure atoms only affect
		#packages from the current repo.

		# Cache the repository-wide package.mask files as a particular
		# repo may be often referenced by others as the master.
		pmask_cache = {}

		def grab_pmask(loc, repo_config):
			if loc not in pmask_cache:
				path = os.path.join(loc, 'profiles', 'package.mask')
				pmask_cache[loc] = grabfile_package(path,
						recursive=repo_config.portage1_profiles,
						remember_source_file=True, verify_eapi=True,
						eapi_default=repo_config.eapi,
						allow_repo=allow_profile_repo_deps(repo_config),
						allow_build_id=("build-id"
						in repo_config.profile_formats))
				if repo_config.portage1_profiles_compat and os.path.isdir(path):
					warnings.warn(_("Repository '%(repo_name)s' is implicitly using "
						"'portage-1' profile format in its profiles/package.mask, but "
						"the repository profiles are not marked as that format.  This will break "
						"in the future.  Please either convert the following paths "
						"to files, or add\nprofile-formats = portage-1\nto the "
						"repository's layout.conf.\n")
						% dict(repo_name=repo_config.name))

			return pmask_cache[loc]

		repo_pkgmasklines = []
		for repo in repositories.repos_with_profiles():
			lines = []
			repo_lines = grab_pmask(repo.location, repo)
			removals = frozenset(line[0][1:] for line in repo_lines
				if line[0][:1] == "-")
			matched_removals = set()
			for master in repo.masters:
				master_lines = grab_pmask(master.location, master)
				for line in master_lines:
					if line[0] in removals:
						matched_removals.add(line[0])
				# Since we don't stack masters recursively, there aren't any
				# atoms earlier in the stack to be matched by negative atoms in
				# master_lines. Also, repo_lines may contain negative atoms
				# that are intended to negate atoms from a different master
				# than the one with which we are currently stacking. Therefore,
				# we disable warn_for_unmatched_removal here (see bug #386569).
				lines.append(stack_lists([master_lines, repo_lines], incremental=1,
					remember_source_file=True, warn_for_unmatched_removal=False))

			# It's safe to warn for unmatched removal if masters have not
			# been overridden by the user, which is guaranteed when
			# user_config is false (when called by repoman).
			if repo.masters:
				unmatched_removals = removals.difference(matched_removals)
				if unmatched_removals and not user_config:
					source_file = os.path.join(repo.location,
						"profiles", "package.mask")
					unmatched_removals = list(unmatched_removals)
					if len(unmatched_removals) > 3:
						writemsg(
							_("--- Unmatched removal atoms in %s: %s and %s more\n") %
							(source_file,
							", ".join("-" + x for x in unmatched_removals[:3]),
							len(unmatched_removals) - 3), noiselevel=-1)
					else:
						writemsg(
							_("--- Unmatched removal atom(s) in %s: %s\n") %
							(source_file,
							", ".join("-" + x for x in unmatched_removals)),
							noiselevel=-1)

			else:
				lines.append(stack_lists([repo_lines], incremental=1,
					remember_source_file=True, warn_for_unmatched_removal=not user_config,
					strict_warn_for_unmatched_removal=strict_umatched_removal))
			repo_pkgmasklines.extend(append_repo(stack_lists(lines), repo.name, remember_source_file=True))

		repo_pkgunmasklines = []
		for repo in repositories.repos_with_profiles():
			if not repo.portage1_profiles:
				continue
			repo_lines = grabfile_package(os.path.join(repo.location, "profiles", "package.unmask"), \
				recursive=1, remember_source_file=True,
				verify_eapi=True, eapi_default=repo.eapi,
				allow_repo=allow_profile_repo_deps(repo),
				allow_build_id=("build-id" in repo.profile_formats))
			lines = stack_lists([repo_lines], incremental=1, \
				remember_source_file=True, warn_for_unmatched_removal=True,
				strict_warn_for_unmatched_removal=strict_umatched_removal)
			repo_pkgunmasklines.extend(append_repo(lines, repo.name, remember_source_file=True))

		#Read package.mask from the user's profile. Stack them in the end
		#to allow profiles to override masks from their parent profiles.
		profile_pkgmasklines = []
		profile_pkgunmasklines = []
		for x in profiles:
			profile_pkgmasklines.append(grabfile_package(
				os.path.join(x.location, "package.mask"),
				recursive=x.portage1_directories,
				remember_source_file=True, verify_eapi=True,
				eapi=x.eapi, eapi_default=None,
				allow_repo=allow_profile_repo_deps(x),
				allow_build_id=x.allow_build_id))
			if x.portage1_directories:
				profile_pkgunmasklines.append(grabfile_package(
					os.path.join(x.location, "package.unmask"),
					recursive=x.portage1_directories,
					remember_source_file=True, verify_eapi=True,
					eapi=x.eapi, eapi_default=None,
					allow_repo=allow_profile_repo_deps(x),
					allow_build_id=x.allow_build_id))
		profile_pkgmasklines = stack_lists(profile_pkgmasklines, incremental=1, \
			remember_source_file=True, warn_for_unmatched_removal=True,
			strict_warn_for_unmatched_removal=strict_umatched_removal)
		profile_pkgunmasklines = stack_lists(profile_pkgunmasklines, incremental=1, \
			remember_source_file=True, warn_for_unmatched_removal=True,
			strict_warn_for_unmatched_removal=strict_umatched_removal)

		#Read /etc/portage/package.mask. Don't stack it to allow the user to
		#remove mask atoms from everywhere with -atoms.
		user_pkgmasklines = []
		user_pkgunmasklines = []
		if user_config:
			user_pkgmasklines = grabfile_package(
				os.path.join(abs_user_config, "package.mask"), recursive=1, \
				allow_wildcard=True, allow_repo=True,
				remember_source_file=True, verify_eapi=False,
				allow_build_id=True)
			user_pkgunmasklines = grabfile_package(
				os.path.join(abs_user_config, "package.unmask"), recursive=1, \
				allow_wildcard=True, allow_repo=True,
				remember_source_file=True, verify_eapi=False,
				allow_build_id=True)

		#Stack everything together. At this point, only user_pkgmasklines may contain -atoms.
		#Don't warn for unmatched -atoms here, since we don't do it for any other user config file.
		raw_pkgmasklines = stack_lists([repo_pkgmasklines, profile_pkgmasklines], \
			incremental=1, remember_source_file=True, warn_for_unmatched_removal=False, ignore_repo=True)
		pkgmasklines = stack_lists([repo_pkgmasklines, profile_pkgmasklines, user_pkgmasklines], \
			incremental=1, remember_source_file=True, warn_for_unmatched_removal=False, ignore_repo=True)
		pkgunmasklines = stack_lists([repo_pkgunmasklines, profile_pkgunmasklines, user_pkgunmasklines], \
			incremental=1, remember_source_file=True, warn_for_unmatched_removal=False, ignore_repo=True)

		for x, source_file in raw_pkgmasklines:
			self._pmaskdict_raw.setdefault(x.cp, []).append(x)

		for x, source_file in pkgmasklines:
			self._pmaskdict.setdefault(x.cp, []).append(x)

		for x, source_file in pkgunmasklines:
			self._punmaskdict.setdefault(x.cp, []).append(x)

		for d in (self._pmaskdict_raw, self._pmaskdict, self._punmaskdict):
			for k, v in d.items():
				d[k] = tuple(v)

	def _getMaskAtom(self, cpv, slot, repo, unmask_atoms=None):
		"""
		Take a package and return a matching package.mask atom, or None if no
		such atom exists or it has been cancelled by package.unmask.

		@param cpv: The package name
		@type cpv: String
		@param slot: The package's slot
		@type slot: String
		@param repo: The package's repository [optional]
		@type repo: String
		@param unmask_atoms: if desired pass in self._punmaskdict.get(cp)
		@type unmask_atoms: list
		@rtype: String
		@return: A matching atom string or None if one is not found.
		"""

		try:
			cpv.slot
		except AttributeError:
			pkg = _pkg_str(cpv, slot=slot, repo=repo)
		else:
			pkg = cpv

		mask_atoms = self._pmaskdict.get(pkg.cp)
		if mask_atoms:
			pkg_list = [pkg]
			for x in mask_atoms:
				if not match_from_list(x, pkg_list):
					continue
				if unmask_atoms:
					for y in unmask_atoms:
						if match_from_list(y, pkg_list):
							return None
				return x
		return None


	def getMaskAtom(self, cpv, slot, repo):
		"""
		Take a package and return a matching package.mask atom, or None if no
		such atom exists or it has been cancelled by package.unmask.

		@param cpv: The package name
		@type cpv: String
		@param slot: The package's slot
		@type slot: String
		@param repo: The package's repository [optional]
		@type repo: String
		@rtype: String
		@return: A matching atom string or None if one is not found.
		"""

		try:
			cpv.slot
		except AttributeError:
			pkg = _pkg_str(cpv, slot=slot, repo=repo)
		else:
			pkg = cpv

		return self._getMaskAtom(pkg, slot, repo,
			self._punmaskdict.get(pkg.cp))


	def getRawMaskAtom(self, cpv, slot, repo):
		"""
		Take a package and return a matching package.mask atom, or None if no
		such atom exists. It HAS NOT! been cancelled by any package.unmask.

		@param cpv: The package name
		@type cpv: String
		@param slot: The package's slot
		@type slot: String
		@param repo: The package's repository [optional]
		@type repo: String
		@rtype: String
		@return: A matching atom string or None if one is not found.
		"""

		return self._getMaskAtom(cpv, slot, repo)



# test_isjustname.py -- Portage Unit Testing Functionality
# Copyright 2006 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

from portage.tests import TestCase
from portage.dep import isjustname

class IsJustName(TestCase):

	def testIsJustName(self):

		cats = ("", "sys-apps/", "foo/", "virtual/")
		pkgs = ("portage", "paludis", "pkgcore", "notARealPkg")
		vers = ("", "-2.0-r3", "-1.0_pre2", "-3.1b")

		for pkg in pkgs:
			for cat in cats:
				for ver in vers:
					if len(ver):
						self.assertFalse(isjustname(cat + pkg + ver),
						msg="isjustname(%s) is True!" % (cat + pkg + ver))
					else:
						self.assertTrue(isjustname(cat + pkg + ver),
						msg="isjustname(%s) is False!" % (cat + pkg + ver))
